<?PHP

/*
 *	W-AGORA 4.0
 *	-----------
 *	$Id: dbaccess.php,v 1.107 2002/06/03 17:54:38 mdruilhe Exp $
 *	Usage:		include file - Generic Database access functions 
 *	Authors:	Marc Druilhe <mdruilhe@w-agora.com>
 *				Tom deLombarde <tomd@blackflute.com>
 *				Franky <liedekef@pandora.be>
 *				Raptor <XpL_Raptor@gmx.net>
 *				Ralf Schaefer <ralf.schaefer@gmx.ch>
 *		... and others
 */

// if (!defined("_DBACCESS")) {
if (!isset($DBACCESS)) {

define("_DBACCESS", 1);
$DBACCESS=1;

define ("ERR_EXISTS", -1);
define ("ERR_NOTARRAY", -2);
define ("ERR_ACCESS", -3);
define ("ERR_NOTFOUND", -4);
define ("ERR_BADARG", -5);

class DBaccess {

	/* public: result array and current row number */
	var $Record   = array();
	var $Row;

	/* public: current error number and error text */
	var $Errno    = 0;
	var $Error    = "";

	var $debug = 0; 			// 0 = turns debug mode off
	var $Halt_On_Error = "yes"; // see setHaltOnError()

	var $children = array();	// array of keys
	var $entries = array();		// array in which notes are stored
	var $users = array();		// loads threads contributors here
	var $lineNo;

	var $autocommit = 1;

	var $autoID = array();


# ------------------------------
# General functions (see PHPLIB)
# ------------------------------

/* private: error handling */
function halt($msg) {
	if ($this->Halt_On_Error == "no")
		return;

	$reason =  $this->Error;
	if (!empty($this->Errno) ) {
	    $reason = '['.$this->Errno.'] '.$reason;
	}

	print '</td></tr></table></td></tr></table></td></tr></table>';
	if ($this->Halt_On_Error == "report") {
		printf(MSG_WARNING, $msg, $reason);
	} else {
		printf(MSG_ERROR, 'Database', $msg, $reason);
	}

	if ($this->Halt_On_Error != "report")
		die(MSG_SESSION_HALTED);
}

/* public: perform a query */
function query($Query_String) {
	echo "function <b>query()</b> not implemented<br>";
	return 0;
}

/* public: walk result set */
function next_record() {
	echo "function <b>next_record()</b> not implemented<br>";
	return 0;
}

/* public: position in result set */
function seek($pos = 0) {
	echo "function <b>seek()</b> not implemented<br>";
}

function affected_rows() {
	echo "function <b>affected_rows()</b> not implemented<br>";
}

function num_rows() {
	echo "function <b>num_rows()</b> not implemented<br>";
}

function num_fields() {
	echo "function <b>num_fields()</b> not implemented<br>";
}

/* public: shorthand notation */
function nf() {
	return $this->num_rows();
}

function np() {
	print $this->num_rows();
}

function f($Name) {
	return $this->Record[$Name];
}

function p($Name) {
	print $this->Record[$Name];
}

function setDebug ($flag) {
	 $this->debug=$flag;
}

/**
 * Print out an error message only if Debug mode is set
 *
 * @param	string	$msg	The message  
 * @since	4.1.2
 * @access	private
 * @return	void
 */
function printDebug($msg) {
	if ($this->debug) print "Debug: <tt>$msg</tt><br>";
} // end func

/**
 * Set the "Halt On Error" flag and returns the old value
 *
 * @param  string	$value	possible values: "yes" (halt with message), "no" (ignore errors quietly), "report" (ignore error, but spit a warning)
 * @since     4.1.2
 * @access    public
 * @return    boolean	The old value
 */
function setHaltOnError ($value="yes") {
	$old = $this->Halt_On_Error;
	$this->Halt_On_Error = $value;
	return $old;
}

#	------------------------------------------------------------------
#				General database access functions
#	------------------------------------------------------------------

function nextId ($seqname, $table="") {
	if (empty ($this->autoID[$seqname])) {
	    $this->autoID[$seqname] = time();
	} else {
	    $this->autoID[$seqname] += 1;
	}
	return $this->autoID[$seqname];
}

/**
 * return the auto-generated ID or sequence number returned by the last INSERT query performed
 * or by the last call to nextId()
 * @returns		the id of the last sequence number
 */
function lastId($seqname, $table="") {
	return $this->autoID[$seqname];
}

function setAutoCommit ($mode=1) {
	$this->autocommit = $mode;
	return TRUE;
}

function commit () {
	return TRUE;
}

function rollback () {
	return TRUE;
}

function openDB ($dbhost="", $dbport="", $dbuser="", $dbpassword="", $dbname="agora", $site="", $persistent=0) {
	// not currently implemented for DBM
	$this->site = $site;
	return 0;
}

function closeDB () {
	return TRUE;
}

function createTable ($table, $field_defs) {
	 echo "function <b>createTable()</b> not implemented<br>";
}

function deleteTable ($table) {
	$ret = $this->query ("DROP TABLE $table");
	if (!$ret) {
		return -3;
	}
	return 0;
}

function updateTable ($table, &$fields, $where) {
	 echo "function <b>updateTable()</b> not implemented<br>";
}

function preserveQuotes($str) {
	return addslashes($str);
}

/**
 * Add a new column (field) in the specified table
 * 
 * Currently, doesnt handle AUTO-increment
 * @param	string	$table	the table where the field is added
 * @param	string	$col	the table where the field is added
 * @param	string	$size	The field definition : INT, TEXT, BLOB, nnn (varchar(n) ) 
 * @since	4.1
 * @access	public
 * @see		createTable
 * @return	void
 */
function addField ($table, $col, $size) {

	if (!ereg ("^[[:alnum:]]+$", $col) ) {
		$this->halt("addField(): invalid argument for column name ($col)");
		return -3;
	}

	if (!ereg ("^[[:alnum:]]+$", $size) ) {
		$this->halt("addField(): invalid argument for field description ($size)");
		return -3;
	}
	
	if (ereg ("^[0-9+]+$", $size) ) {
		if ($size>255) {
			$desc = 'TEXT';
		} else {
			$desc = "VARCHAR($size) NOT NULL DEFAULT ''";
		}
	} else {
		$type = strtoupper($size);
		switch ($type) {
			case "INT":
				$desc = "INT NOT NULL DEFAULT 0";
				break;
			default:
				$desc = $type;
				break;
		}
	}

	$ret = $this->query ("ALTER TABLE $table ADD $col $desc");
}

/**
 * drop a column (field) from the specified table
 *
 * @param	string	$table	the table where the field must be deleted
 * @param	string	$col	the name of the field
 * @since	4.1
 * @access	public
 * @see		addField
 * @return	void
 */
function dropField ($table, $col) {
	echo "function <b>dropField()</b> not implemented<br>";
}

#	------------------------------------------------------------------
#				Site management functions
#	------------------------------------------------------------------

function createSite ($site, $descr_file) {

# reads fields description
# -------------------------
	if (!is_file ($descr_file)) {
		echo "error: createSite(): could not include file: '$descr_file'"; 
		return -4;
	}

	include "$descr_file";

	$field_descr =array();

# Set core variables (managed by w-agora)
# ---------------------------------------
	if (is_array ($bn_var)) {
		reset ($bn_var);
		while (list(, $var) = each($bn_var)) {
			$field_descr[$var] = $bn_var_size[$var];
		}
	}

# add user fields
# ---------------
	if (is_array ($db_var)) {
		reset ($db_var);
		while (list(, $var) = each($db_var)) {
			$field_descr[$var] = $bn_var_size[$var];
		}
	}

# add DB management fields
# ------------------------
	$field_descr["cle"] = "INT";
	$field_descr["parent"] = "INT";
	$field_descr["childs"] = "INT";
	$field_descr["thread"] = "INT";
	$field_descr["newest"] = "INT";

	return $this->createTable ($forum, $field_descr);
}

function deleteSite ($site) {

# remove forums table
# -------------------
	$this->Halt_On_Error = "report";
	$ret = $this->deleteTable ($site);
	if ($ret != 0) {
		return $ret;
	}

# remove users table
# ------------------
	$ret = $this->deleteTable ("${site}_users");
	if ($ret != 0) {
		return $ret;
	}

# remove permissions table
# ------------------------
	$ret = $this->deleteTable ("${site}_userforum");
	if ($ret != 0) {
		return $ret;
	}

# remove user-thread subscription table
# -------------------------------------
	$ret = $this->deleteTable ("${site}_userthread");
	if ($ret != 0) {
		return $ret;
	}

# remove attachments table
# ------------------------
	$ret = $this->deleteTable ("${site}_attachments");
	if ($ret != 0) {
		return $ret;
	}

# remove LOG table
# ----------------
	$ret = $this->deleteTable ("${site}_log");
	if ($ret != 0) {
		return $ret;
	}

	return 0;
}

#	------------------------------------------------------------------
#				LOG / STATS functions
#	------------------------------------------------------------------


/**
 * Gets monthly statistics.
 *
 * @param	site	the site 
 * @param	year	the year (YYYY) facultative
 * @param	month	the month (1-12) facultative (if set, year must also be set)
 * @since	4.1.3
 * @access	public
 * @return	array		A double dimension array : $array[month][action] = count 
 *						where month as the form YYYYMM, and action 
 *						can be one value of view|insert|subscribe|login
 */
function getMonthlyStats ($site, $year='', $month='') {
	$log_table = $site.'_log';

	$query = "SELECT YEAR(datestamp) AS y, MONTH(datestamp) AS m, action, count(action) AS cnt FROM $log_table";
	
	if (!empty($year)) {
		$y = sprintf('%04d', $year);
	    $query .= " WHERE year(datestamp)='$y'";
		if (!empty($month)) {
			$m = sprintf('%02d', $month);
			$query .= " AND month(datestamp)='$m'";
		}
	}

	$query .= ' GROUP BY YEAR(datestamp), MONTH(datestamp), action';

	$this->query($query);
	while ($this->next_record()) {
		$m = sprintf ('%04d%02d', $this->Record['y'], $this->Record['m']);
		$action = $this->Record['action'];
		$stats[$m][$action] = (int) $this->Record['cnt'];
	}
	return $stats;
}

/**
 * Gets monthly visitors.
 *
 * Returns an array indexed by months with the number of distinct visitors 
 * @param	site	the site 
 * @since	4.1.3
 * @access	public
 * @return	array		An array of the form: $array[month] = visits where month as the form YYYYMM
 */
function getMonthlyVisitors ($site, $year='', $month='') {
	$log_table = $site.'_log';

	$query = "SELECT YEAR(datestamp) AS y, MONTH(datestamp) AS m, count(DISTINCT userid) as visits FROM $log_table";

	if (!empty($year)) {
		$y = sprintf('%04d', $year);
	    $query .= " WHERE year(datestamp)='$y'";
		if (!empty($month)) {
			$m = sprintf('%02d', $month);
			$query .= " AND month(datestamp)='$m'";
		}
	}

	$query .= ' GROUP BY YEAR(datestamp), MONTH(datestamp), action';
	$this->query($query);
	while ($this->next_record()) {
		$m = sprintf ('%04d%02d', $this->Record['y'], $this->Record['m']);
		$visits[$m] = (int) $this->Record['visits'];
	}
	return $visits;
}

/**
 * Gets Daily statistics.
 *
 * @param	site	the site 
 * @param	year	the year (YYYY) 
 * @param	month	the month (1-12)
 * @since	4.1.3
 * @access	public
 * @return	array		A double dimension array : $array[day][action] = count 
 *						where day is in the rang 1..31 and action can be one value of view|insert|subscribe|login
 */
function getDailyStats ($site, $year, $month) {

	$log_table = $site.'_log';
	$y = sprintf('%04d', $year);
	$m = sprintf('%02d', $month);

	$this->query("SELECT dayofmonth(datestamp) AS d, action, count(action) AS cnt FROM $log_table WHERE year(datestamp)='$y' AND month(datestamp)='$m' GROUP BY d, action");
	while ($this->next_record()) {
		$d = sprintf('%02d', $this->Record['d']);
		$action = $this->Record['action'];
		$stats[$d][$action] = (int) $this->Record['cnt'];
	}
	
	return $stats;
}

/**
 * Gets Daily statistics.
 *
 * @param	site	the site 
 * @param	year	the year (YYYY) 
 * @param	month	the month (1-12)
 * @since	4.1.3
 * @access	public
 * @return	array		A double dimension array : $array[day][action] = count 
 *						where day is in the rang 1..31 and action can be one value of view|insert|subscribe|login
 */
function getDailyVisitors ($site, $year, $month) {

	$log_table = $site.'_log';
	$y = sprintf('%04d', $year);
	$m = sprintf('%02d', $month);

	$this->query("SELECT dayofmonth(datestamp) AS d, count(DISTINCT userid) as visits FROM $log_table WHERE year(datestamp)='$y' AND month(datestamp)='$m' GROUP BY d");
	while ($this->next_record()) {
		$d = sprintf('%02d', $this->Record['d']);
		$visits[$d] = (int) $this->Record['visits'];
	}
	return $visits;
}

/**
 * Insert an entry in the LOG table 
 */
function addLogEntry ($forum, $action, $userid, $thread=0, $key=0, $ip='') {
	$log_table = $this->site.'_log';
	$ip = ($ip=='') ? get_remote_ip() : $ip;
	$this->query ("INSERT INTO $log_table (userid, forum, action, thread, note_id, ip) VALUES ('$userid', '$forum', '$action', '$thread', '$key', '$ip')");
} // end func


function getSiteStats ($site) {
	$result = $this->query ("SELECT count(bn_name) as totalforums, sum(totalthreads) as totalthreads, sum(totalnotes) as totalnotes, sum(att_count) as totalfiles, sum(att_size) as totalfilesize, max(lastnote) as lastnote FROM $site WHERE state != '0' AND rank != '0' AND category=0");

	if ($this->next_record() ) {
		$stats['totalforums']   = (int) $this->f('totalforums');
		$stats['totalthreads']  = (int) $this->f('totalthreads');
		$stats['totalnotes']    = (int) $this->f('totalnotes');
		$stats['totalfiles']    = (int) $this->f('totalfiles');
		$stats['totalfilesize'] = (int) $this->f('totalfilesize');
		$stats['lastnote'] = (int) $this->f('lastnote');
	} 

	$result = $this->query ("SELECT count(userid) as totalusers FROM ${site}_users WHERE lastlogin>0");
	if ($this->next_record() ) {
		$stats['totalusers'] = $this->f('totalusers');
	}

	return $stats;
}

/**
 * Get Forum statistics.
 *
 * @param	forum	the forum table ($bn_db)
 * @since	4.1.3
 * @access	public
 * @return	array	totalhits = # views, totalauthors = # of distincts authors
 */
function getForumStats ($forum) {

	$q = "SELECT sum(hits) as totalhits, count(DISTINCT userid) as totalauthors FROM $forum WHERE hidden='0'";
	$this->query("$q");
	if ($this->next_record()) {
		return $this->Record;
	}

	return false;
}


#	------------------------------------------------------------------
#				Forum management functions
#	------------------------------------------------------------------

function createForum ($forum, $descr_file) {

# reads fields description
# -------------------------
	if (!is_file ($descr_file)) {
		echo "error: createForum(): could not include file: '$descr_file'"; 
		return -4;
	}

	include "$descr_file";

	$field_descr =array();

# Set core variables (managed by w-agora)
# ---------------------------------------
	if (is_array($bn_var) && isset($admin_table) ) {
		// $bn_var set in admin templates
		reset ($bn_var);
		while (list(, $var) = each($bn_var)) {
			$field_descr[$var] = $bn_var_size[$var];
		}
	} else {
		// set default w-agora variables for a new forum
		$field_descr["unixdate"]	= "INT";
		$field_descr["userid"]		= "32";
		$field_descr["mod_date"]	= "INT";
		$field_descr["mod_userid"]	= "32";
		$field_descr["password"]	= "32";
		$field_descr["filename"]	= "64";
		$field_descr["att_size"]	= "INT";
		$field_descr["mail_reply"]	= "1";
		$field_descr["hidden"]		= "INT";
		$field_descr["hits"]		= "INT";
		$field_descr["closed"]		= "INT";
		$field_descr["ip"]			= "32";
	}

# add user fields
# ---------------
	if (is_array ($db_var)) {
		reset ($db_var);
		while (list(, $var) = each($db_var)) {
			$field_descr[$var] = $bn_var_size[$var];
		}
	}

# add DB management fields
# ------------------------
	$field_descr["cle"] = "INT";
	$field_descr["parent"] = "INT";
	$field_descr["childs"] = "INT";
	$field_descr["thread"] = "INT";
	$field_descr["newest"] = "INT";

	return $this->createTable ($forum, $field_descr);
}

function deleteForum ($site, $forum) {

# delete table
	$ret = $this->deleteTable ("$forum");
	if ($ret != 0) {
		$this->Error ="could drop table $forum from site $site";
		return $ret;
	}

# Delete entry in the forums table ($site)
# ---------------------------------------
	$ret = $this->deleteForumEntry ($site, $forum);
	if ($ret < 0) {
		$this->Error ="could not delete entry for forum $forum in table $site";
		return -3;
	}

# delete attachments (table $site_attachments) (files are removed outside this function)
# --------------------------------------------------------------------------------------
	$this->query ("DELETE FROM ${site}_attachments WHERE bn_name='$forum'");

# delete all user->forum relations (table $site_userforum)
# --------------------------------------------------------
	$this->deletePrivs ($site, $forum, '');

# Delete user subscriptions for this forum (table $site_userthread)
# -----------------------------------------------------------------
	$this->unSubscribeUser ($site, $forum);

	return 0;
}

function insertForum($site, &$fields) {
	global $now;

# check if forum already exists
	$forum = $fields["bn_name"];
	$result = $this->query ("SELECT bn_name FROM $site WHERE bn_name='$forum'");
	if ($this->next_record() ) {
		return ERR_EXISTS;
	}

	if (empty($fields["parent"])) {
		$fields["parent"] = 0;
	}
	$fields["thread"] = $now;
	$fields["childs"] = 0;
	if (empty($fields["unixdate"])) {
		$fields["unixdate"] = $now;
	}
	$fields["newest"] = $fields["unixdate"];
	$fields["cle"] = $now;
	$fields["category"] = 0;
	$ret = $this->insertRow($site, $fields);
	return ($ret==0) ? $now : $ret;
}

function deleteForumEntry ($site, $forum) {
	$ret = $this->query ("DELETE FROM $site WHERE bn_name='$forum'");
	if (!$ret) {
		$this->Error ="could not delete entry for forum $forum in table $site";
		return -3;
	}
	return 0;
}

# gets info about forum $name in site $site 
# -----------------------------------------
function getForum ($site, $name) {
	
	/*
	$halt = $this->Halt_On_Error;
	$this->Halt_On_Error = "no";

	$this->query ("SELECT parent AS cat FROM $site WHERE bn_name='$name'");
	if ($this->next_record() ) {
	    $cat = $this->Record["cat"];
		$this->query ("SELECT bn_title FROM $site WHERE cle='$cat'");
		if ($this->next_record() ) {
			$cat_title = $this->Record["bn_title"];
		}
	}
	$result = $this->query ("SELECT S.*, U.username as ownername, U.useraddress as owneraddress FROM $site S, ${site}_users U WHERE S.bn_name='$name' AND U.userid = S.owner");

	$this->Halt_On_Error = $halt;
	if ($this->next_record() ) {
		$this->Record["cat_id"] = $this->Record["parent"];
		$this->Record["cat_title"] = $cat_title;
		return $this->Record;
	} else {
		return -3;
	}
	$this->Halt_On_Error = $halt;

	*/

	$result = $this->query ("SELECT S.parent AS cat_id, S.*, C.bn_title AS cat_title, U.username AS ownername, U.useraddress AS owneraddress FROM $site S LEFT JOIN ${site}_users U ON (S.owner = U.userid) LEFT JOIN $site C ON (C.cle=S.parent) WHERE S.bn_name='$name'");

	if ($this->next_record() ) {
		return $this->Record;
	} else {
		return -3;
	}
}

function updateForum ($site, $forum, $fields) {
	$result =  $this->query ("SELECT bn_name FROM $site WHERE bn_name='$forum'");
	if (!$result) { /* access problem */
		return -3;
	}

	if (!$this->next_record() ) {
		return -4; /* no row selected */
	}

	$fields["bn_name"] = $forum;
	return $this->updateTable ($site, $fields, "bn_name='$forum'");
}

function updateForumStats ($site, $forum) {

	if (!$this->debug) {
		$this->Halt_On_Error = "no";
	}
	
	$fields = array();

	$result = $this->query ("select count(*) as totalnotes, max(unixdate) as lastnote from $forum where hidden=0");
	if (!$this->next_record() ) {
		return -1;
	}

	$fields["totalnotes"] = $this-> f("totalnotes");
	$fields["lastnote"] = $this-> f("lastnote");

	$result = $this->query ("select count(*) as totalthreads from $forum where parent=0 and hidden=0");
	if (! $this->next_record() ) {
		return -1;
	}
	$fields["totalthreads"] = $this-> f("totalthreads");

	$result = $this->query ("SELECT count(att_size) AS att_count, sum(att_size) AS att_size FROM ${site}_attachments WHERE bn_name='$forum' AND hidden!='1'");
	if ($this->next_record() ) {
		$fields["att_count"] = (int) $this-> f("att_count");
		$fields["att_size"] = (int) $this-> f("att_size");
	}

	$ret = $this->updateTable ($site, $fields, "bn_db='$forum'");

	$this->Halt_On_Error = "yes";
	return $ret;
}

function getForums ($site, $moder="", $cat=0) {

	$select = "S.bn_name, S.bn_title";
	$where = "S.state != '0' AND S.rank != '0' AND S.category=0";
	if (!empty($cat) ) {
	    $where .= " AND S.parent='$cat'";
	}
	$sort = "S.parent, S.bn_title";

	if (empty($moder)) {
		$from = "$site S";
	} else {
		$from = "$site S, ${site}_userforum UF";
		$where .= " AND S.bn_name=UF.bn_name AND UF.userid = '" . $this->preserveQuotes ($moder) ."' AND UF.modpriv=1";
	}

	$result = $this->query ("SELECT $select FROM $from WHERE $where ORDER BY $sort");
	while ($this->next_record() ) {
		$name = $this->f("bn_name");
		$forums[$name] = $this->f("bn_title");
	}
	return $forums;
}

/**
 * First version, only one category level
 */
function listForums ($site, $moder="", $sort="", $hide_inactive=0, $cat='') {
	global $auth;
	
	if ($site=="agora") {
		$query = "SELECT S.*, 1 as catorder, U.username as ownername, U.useraddress as owneraddress FROM $site S, ${site}_users U WHERE U.userid = S.owner";
	} elseif (empty($moder)) {
		// $query = "SELECT S.*, C.rank AS catorder, U.username as ownername, U.useraddress as owneraddress FROM $site S, $site C, ${site}_users U WHERE S.parent=C.cle AND S.category!=1 AND U.userid = S.owner";

		$query = "SELECT S.*, C.rank AS catorder, U.username AS ownername, U.useraddress AS owneraddress FROM $site S LEFT JOIN ${site}_users U ON (S.owner = U.userid) LEFT JOIN $site C ON (C.cle=S.parent) WHERE S.category!=1";
	} else {
# gets all forums for wich $moder is moderator
		$query = "SELECT S.*, C.rank AS catorder, U.username as ownername, U.useraddress as owneraddress FROM $site S, $site C, ${site}_users U, ${site}_userforum UF WHERE S.parent=C.cle AND S.category!=1 AND S.bn_name=UF.bn_name AND UF.userid = '" . $this->preserveQuotes($moder)."' AND UF.modpriv=1";
	}

	if ($hide_inactive) {
		$query .= " AND S.state != '0' AND S.rank != '0'";
	}

	if ($cat != '') {
	    $query .= " AND S.parent='$cat'";
	}

	if (empty($sort)) {
		$order = ",S.bn_title";
	}else {
	    $a=explode (",",$sort);
		while ( list(,$s)=each ($a) ) {
		    $order .= ", S.".trim($s);
		}
	}
	$query .= " ORDER BY catorder, S.rank". $order;
	$result = $this->query ($query);
	reset ($this->entries);
	while ($this->next_record() ) {
		$name=ereg_replace ("^${site}_", "", $this->Record["bn_name"]);
		$forums[$name] = $this->Record;
	}

	if (!is_array($forums)) {
	    return false;
	}

# Hide private forums for unauthorized users
	$showpriv = false;
	if ($site=="agora") {
		$showpriv =true;
	} elseif (empty($auth->userid) ) {
		$list_access =array();
	} elseif ($auth->level < ADMIN) {
		$u = $this->preserveQuotes($auth->userid);
		$r = $this->query ("SELECT bn_name FROM ${site}_userforum WHERE userid='$u' AND listpriv='1'");
		while ($this->next_record() ) {
			$n=$this->Record["bn_name"];
			$list_access[$n]=1;
		}
	} else {
		$showpriv =true;
	}

# Gets all forums that the user can list 
	unset ($this->entries);
	reset ($forums);
	while (list($n,$f) = each($forums) ) {
		$bn = $f["bn_name"];
		if ( ($f["type"]!="priv") || $showpriv || isset($list_access[$bn]) ) {
			$key = $f["cle"];
			$this->entries[$key] = $f;
			$this->children[0][] = $key;
		} else {
			unset ($forums[$n]);
		}
	}
	return $forums;
}

function displayForumList () {
	global $site, $bn, $ext, $main_page, $cookies, $auth, $is_authenticate;
	global $site_totalthreads, $site_totalnotes, $site_totalforums;
	global $site_title, $login_string;
	global $admin_url, $admin_text, $admin_link;
	global $list_url, $list_text, $list_link;
	global $post_url, $post_text, $post_link;
	global $search_url, $search_text, $search_link;
	global $login_url, $login_text, $login_link;
	global $register_url, $register_text, $register_link;
	global $profile_url, $profile_text, $profile_link;
	global $subscribe_url, $subscribe_text, $subscribe_link;
	global $download_forum_url, $download_forum_text, $download_forum_link;
	global $forum_list_url, $forum_list_text, $forum_list_link;
	global $choose_forum_box, $choose_forum_form;
	global $bn_win_width, $bn_win_height;
	global $forum_types;

	if (!is_array($this->entries)) return -1;
	if (!is_array($this->children[0])) return -1;

	// displays list header
	$l = str_replace('"', '\"', $GLOBALS["bn_list_header"])."\n";
	eval ("echo \"$l\";");
	// eval ("echo stripSlashes(\"$l\");");

	$GLOBALS["bn_list_category"] = str_replace('"','\"', $GLOBALS["bn_list_category"])."\n";
	$GLOBALS["bn_list_odd_first_line"] = str_replace('"','\"', $GLOBALS["bn_list_odd_first_line"])."\n";
	$GLOBALS["bn_list_even_first_line"] = str_replace('"','\"', $GLOBALS["bn_list_even_first_line"])."\n";

	$categories = $this->getCategories($site);

## display all forums
## ------------------
	$curcat = 0;	// current displayed category
	$this->lineNo = 0;
	reset($this->children[0]);
	while ( list (,$key) = each ($this->children[0]) ) {
		// evaluate all variables
		extract($this->entries[$key]);

# Handle cookies / new stuff
# --------------------------
		$new_icon = "";
		$is_updated = FALSE;
		if (isset($GLOBALS["WAS_${bn_name}"]) ) {
			parse_str($GLOBALS["WAS_${bn_name}"]);
			if ($lastnote > $ThisVisit) {
				$is_updated = TRUE;
				$new_icon=msg(82);
			}
		} elseif (isset($GLOBALS["WAP_${bn_name}"]) ) {
			parse_str($GLOBALS["WAP_${bn_name}"]);
			if ($lastnote > $LastVisit) {
				$is_updated = TRUE;
				$new_icon=msg(82);
			}
		}

# set updated color
# -----------------
		if ($lastnote > 0) {
			$age = $GLOBALS[now] - $lastnote;
			if ($age > ( (int) $GLOBALS['bn_new_age']) ) {
				$date_color = $GLOBALS['bn_normal_color'];
			} elseif ($age < ( (int) $GLOBALS['bn_hot_age']) ) {
				$date_color = $GLOBALS['bn_hot_color'];
			} else {
				$date_color = $GLOBALS['bn_new_color'];
			}
			$lastnote = formated_date ($lastnote);
		} else {
			$lastnote = "";
		}

		$formated_date = formated_date($unixdate);
		$moder_icon = ($moderated == 1) ? YES : NO;
		$access_icon = $forum_types[$type];

		switch ($state) {
		 case "0":	// inactive
			$state_icon=msg(130,"<img src='images/locked_thread.gif' alt='inactive forum'>");
			break;
		 case "L":	// locked
			$state_icon=msg(131,"<img src='images/locked_thread.gif' alt='locked forum'>");
			break;
		 case "C":	// closed
			$state_icon=msg(146,"<img src='images/locked_thread.gif' alt='closed forum'>");
			break;
		 case "1":	// active
			if ($is_updated) {
				$state_icon=msg(148,"<img src='images/new_thread.gif' alt='new messages'>");
			} else {
				$state_icon=msg(132,"<img src='images/thread.gif' alt='active forum'>");
			}
			break;
		}
		$active_icon = $state_icon;

		$cat_id = $parent;
		if ($cat_id != $curcat) {
			// new category
			$cat_title = $categories[$cat_id];
			$cat_list_url = "list.$ext?site=$site&cat=$cat_id";
			$curcat = $cat_id;
			$l = $GLOBALS["bn_list_category"];

			if (!empty($GLOBALS["bn_before_line"]) ) {
				eval ($GLOBALS["bn_before_line"]);
			}
			eval ("echo stripSlashes(\"$l\");");
		}

		// display line
		$line = ($this->lineNo % 2) ? "bn_list_odd_first_line" : "bn_list_even_first_line";
		if (!empty($GLOBALS[$line]) ) {
			$this->lineNo ++;

			if (!empty($GLOBALS["bn_before_line"]) ) {
				eval ($GLOBALS["bn_before_line"]);
			}
			$l = $GLOBALS[$line];
			eval ("echo stripSlashes(\"$l\");");
		}
	}

	// displays list footer
	$l = str_replace('"', '\"', $GLOBALS["bn_list_footer"])."\n";
	eval ("echo stripSlashes(\"$l\");");
}


#	------------------------------------------------------------------
#				Categories functions
#	------------------------------------------------------------------

function getCategories ($site) {

	$query = "SELECT cle, rank, bn_title FROM $site WHERE category=1 ORDER BY rank, bn_title";
	$result = $this->query ($query);
	while ($this->next_record() ) {
		$cat_id = $this->Record["cle"];
		$cats[$cat_id] = $this->Record["bn_title"];
	}
	return $cats;
}

function addCategory ($site, $cat, $parent=0) {
	global $now;

	$fields["cle"]		= $now;
	$fields["thread"]	= $now;
	$fields["unixdate"]	= $now;
	$fields["newest"]	= $now;
	$fields["parent"]	= (int) $parent;
	$fields["childs"]	= 0;
	$fields["bn_title"]	= $cat["title"];
	$fields["rank"]		=  $cat["order"];
	$fields["owner"]	= $cat["owner"];
	$fields["category"] = "1";
	$fields["state"]	= "1";
	$fields["category"] = "1";
	// $fields["bn_name"]	= "category";
	$ret = $this->insertRow($site, $fields);
	return ($ret==0) ? $now : $ret;
}

function deleteCategory ($site, $key) {
	$ret = $this->query ("DELETE FROM $site WHERE cle='$key'");
	if ($ret) {
		// Change category for forums that were in this category
		$f["parent"] = "0";
		return $this->updateTable ("$site", $f, "parent='$key'");
	}

}

function getCategory ($site, $key) {
	$ret = $this->query ("SELECT bn_title AS cat_title, rank, state, owner FROM $site WHERE cle='$key'");
	if ($this->next_record() ) {
		return $this->Record;
	} else {
	    return -3;
	}

}

#	------------------------------------------------------------------
#				User management functions
#	------------------------------------------------------------------

function getUser($site, $value, $field="userid") {

	if (empty($value)) {
	    return ERR_BADARG;
	}
	$query = sprintf ("SELECT * FROM ${site}_users WHERE $field='%s'", $this->preserveQuotes($value) );
	$result = $this->query ($query);
	if ($this->next_record() ) {
		return $this->Record;
	} else {
		return -3;
	}
}

/**
 * reads a user from the db and chacks their password.
 * if passwords don't match it returns nothing.
 */
function getUserCheckPassword($site, $userid, $password) {

	$user =$this->getUser($site, $userid);
	if (is_array($user) && ($password != $user["password"]) )
		return;
	else
		return $user;
}

/**
 * returns the latest registered user from the site user database
 *
 * @author	Raptor
 * @param	string 	$site		The site where to find the user
 * @returns	array	"userid"	The user's id
 *			"username"	The user's name
 *			"useraddress"	The user's email address
 */
function getNewestUser($site) {
	$this->query ("SELECT max(unixdate) AS lastcle FROM ${site}_users WHERE lastlogin>0");
	if (!$this->next_record() ) {
		return -3;
	}

	$lastcle = $this->Record["lastcle"];
	$this->query ("SELECT userid, username, useraddress FROM ${site}_users WHERE unixdate='$lastcle'");
	if ($this->next_record() ) {
		$u = array (
			'userid'      => $this->Record["userid"],
			'username'    => $this->Record["username"],
			'useraddress' => $this->Record["useraddress"]
		);
		return $u;
	} else {
		return -3;
	}
}

/**
 *	get all site administrators (userid, username, useraddress) into an array
 *
 */
function getAdministrators ($site) {

	$users = array();

	// get all administrators
	$query = "SELECT userid, username, useraddress FROM ${site}_users WHERE userpriv='admin' OR userpriv='root'";

	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

	$i=0;
	while ($this->next_record() ) {
		$u = $this->Record["userid"];
		$users[$u] = $this->Record;
		$i++;
	}

	return ($i>0) ? $users : 0;
}

/**
 *	get all moderators having moderator privilege on a given forum 
 *  (userid, username, useraddress) into an array
 */
function getModerators ($site, $forum) {

	$users = array();

	// get all moderators having moderator privilege on this forum
	$query = "SELECT U.userid, U.username, U.useraddress FROM ${site}_users U, ${site}_userforum UF WHERE U.userid=UF.userid AND U.userpriv='moder' AND UF.bn_name='$forum' AND UF.modpriv=1";

	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

	$i=0;
	while ($this->next_record() ) {
		$u = $this->Record["userid"];
		$users[$u] = $this->Record;
		$i++;
	}

	return ($i>0) ? $users : 0;
}

/**
 *	get all users in the site 'site' matching the given criteria
 * 	- if $forum is set, then search only users which have access to this forum
 *		- if $privs is set, then search only users which a given privilege (admin, moder, user)
 *		- if $userid is set, then returns only users where userid begins with $userid
 *		- if $state is set, then search only users in the given state (active, pending, inactive)
 */
function getUsers($site, $forum="", $privs="", $userid="", $state="", $sort="userid") {

	if (!empty($forum) || !empty($state) ) {
		$query = "SELECT U.*, UF.state, UF.bn_name FROM ${site}_users U, ${site}_userforum UF WHERE U.userid=UF.userid";
	} else {
		$query = "SELECT U.* FROM ${site}_users U WHERE U.unixdate>0";
	}

	if (!empty($userid)) {
		$query .= " AND U.userid LIKE '$userid%'";
	}

	if (!empty($privs)) {
		$p = split (',' , $privs);
		$query .= " AND (U.userpriv='" . implode ("' OR U.userpriv='", $p) . "')";
	}

	if (!empty($forum)) {
		$query .= " AND UF.bn_name='$forum'";
		
	}
	
	if ($state != "") {
		$query .= " AND UF.state='$state'";
	}
	
	if (!empty($sort)) {
		$query .= " ORDER BY U.$sort";
	}

	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

	$i=0;
	$users = array();

	while ($this->next_record() ) {
		$u = $this->Record["userid"];
		$users[$u] = $this->Record;
		$i++;
	}

	return ($i>0) ? $users : 0;
}


/**
 *	get all users in the site 'site' 
 *	@returns	array	$users	associative array (pairs userid/username)
 */
function getAllUsers($site, $sort="username", $where="") {

	$query = "SELECT userid,username FROM ${site}_users";


	if (!empty($where)) {
		$query .= " WHERE $where";
	}

	if (!empty($sort)) {
		$query .= " ORDER BY $sort";
	}

	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

	while ($this->next_record() ) {
		$userid = $this->Record["userid"];
		$users[$userid] = $this->Record["username"];
	}

	return $users;
}

/**
 * get all authors (registered or not) who posted into one or all forums
 *
 * @param	string $site	The site name
 * @param	string $forum	The forum name ($bn). If empty, then all forums will be searched
 * @param	string $where	Optional restrictive where clause
 * @see		getAllUsers()
 * @since	4.1
 * @access	public
 * @return	array	$users	associative array (pairs username/username)
 */
function getAllAuthors($site, $forum="", $where="") {

 # gets all forums => load $forums[] with all tables names in this site
 # -------------------------------------------------------------------
	$query = "SELECT distinct bn_db, bn_name FROM $site WHERE state != '0' AND category=0";
	if (!empty($forum) ) {
		$query .= " AND bn_name='$forum'";
	}

	$result = $this->query ($query);
	while ($this->next_record() ) {
		$table = $this->Record["bn_db"];
		$forums[$table] = $this->Record["bn_name"];
	}

	if (!is_array ($forums) ) {
		return -1;
	}

	reset ($forums);
	while (list($table, $bn) = each ($forums) ) {
		$query = "SELECT DISTINCT username FROM $table";
		if (!empty($where)) {
			$query .= " WHERE $where";
		}

		$result = $this->query ($query);
		while ($this->next_record() ) {
			$username = $this->Record["username"];
			$users[$username] = $this->Record["username"];
		}
	}

	asort ($users);
	return $users;
}

/**
 *	get all users in the site 'site' who post into a forum
 *	@returns	array	$users	associative array (pairs userid/username)
 */
function getActiveUsers($site, $forum="", $sort="username") {

	$query = "SELECT DISTINCT userid,username FROM ${site}_userforum";

	if (!empty($forum)) {
		$query .= " AND bn_name='$forum'";
	}

	if (!empty($sort)) {
		$query .= " ORDER BY $sort";
	}

	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

	while ($this->next_record() ) {
		$userid = $this->Record["userid"];
		$users[$userid] = $this->Record["username"];
	}

	return $users;
}

/**
 * Gets all users that didn't login into the site or didn't post any 
 * message since a given date
 *
 * @param	string $site	The site name
 * @param	string $date	The ending date
 * @param	string $type	type of search (post | login)
 * @param	string $sort	Optional order clause (default userid)
 * @return	array	$users	the users that matched the query
 */
function getInactiveUsers($site, $date, $type="post", $sort="userid") {

# Gets all users according to the action "type" ("last post" or "last login")
# ---------------------------------------------------------------------------
	if ($type=="post") {
		$query = "SELECT * FROM ${site}_users WHERE userpriv='user' AND lastpost < $date AND unixdate < $date";
	} else {
		$query = "SELECT * FROM ${site}_users WHERE userpriv='user' AND lastlogin < $date AND unixdate < $date";
	}

	if (!empty($sort)) {
		$query .= " ORDER BY $sort";
	}

# Gets all users satisfying the query
# -----------------------------------
	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

	$i=0;
	$users = array();
	while ($this->next_record() ) {
		$users[$i++] = $this->Record;
	}

	return ($i==0) ? "" : $users;
}

function insertUser($site, &$fields) {
	global $now;

	if (!is_array($fields) || empty($fields['userid'])) {
		echo "warning: insertUser(): invalid argument";
	    return ERR_BADARG;
	}

	if (empty($fields["unixdate"])) {
		$fields["unixdate"] = $now;
	}
 
	 // !!! only for compatibility purpose (should be removed, but was set as primary index in older versions)
	$fields["cle"] = $now; 

	$userid=$this->preserveQuotes($fields['userid']);

	$result = $this->query ("SELECT userid FROM ${site}_users WHERE userid='$userid'");
	if ($this->next_record() ) {
		return -3;		// key already exists, try later...
	}

	$ret = $this->insertRow("${site}_users", $fields);
	return ($ret==0) ? $now : $ret;
}

function deleteUser ($site, $userid) {

	$userid=$this->preserveQuotes($userid);

# Unsubscribe this user from receiving emails (to be done before removing him)
	$result = $this->query ("SELECT useraddress FROM ${site}_users WHERE userid='$userid'");
	if ($this->next_record() ) {
		$useraddress = $this->f("useraddress");
		$this->unSubscribeUser ($site, "", $useraddress, 0);
	}

# delete user in users table
	$result = $this->query ("DELETE FROM ${site}_users WHERE userid='$userid'");
	if (!$result) {
		return -3;
	}

# delete privileges in userforum table
	$result = $this->query ("DELETE FROM ${site}_userforum WHERE userid='$userid'");
	if (!$result) {
		return -3;
	}

# remove 'owner' field for all forums owned by this user 
	$result = $this->query ("UPDATE $site SET owner='admin' WHERE owner='$userid'");
	if (!$result) {
		return -3;
	}

	return 0;
}

function updateUser($site, $userid, &$fields) {

	$query = sprintf ("SELECT userid,useraddress FROM ${site}_users WHERE userid='%s'", $this->preserveQuotes($userid) );
	$result =  $this->query ($query);
	if (!$result) { /* access problem */
		return -3;
	}

	if (!$this->next_record() ) {
		return -4; /* no row selected */
	}

	if (isset($fields["useraddress"]) && ($site!= "agora") ) {
		$newaddress = $fields["useraddress"];
		$useraddress = $this->f("useraddress");
		if ($useraddress != $newaddress) {
			$result =  $this->query ("UPDATE ${site}_userthread SET useraddress='$newaddress' WHERE useraddress='$useraddress'");
		}
	}
		
	return $this->updateTable ("${site}_users", $fields, "userid='".$this->preserveQuotes($userid)."'");
}

function updateUserStats($site, $forum, $userid, $lastpost) {

	$result =  $this->query ("SELECT userid FROM ${site}_userforum WHERE userid='$userid' AND bn_name='$forum'");
	if (!$result) { /* access problem */
		return -3;
	}

	if ($this->next_record() ) {
		$query = "UPDATE ${site}_userforum SET totalpost=totalpost+1, lastpost=$lastpost WHERE userid='$userid' AND bn_name='$forum'";
	} else {
		$query = "INSERT INTO ${site}_userforum (userid, bn_name, listpriv, readpriv, writepriv, totalpost, lastpost, state) VALUES('$userid', '$forum', '1',  '1', '1', '1', '$lastpost', '1')";
	}
	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

	$query = "UPDATE ${site}_users SET totalposts=totalposts+1, lastpost=$lastpost, lastforumpost='$forum' WHERE userid='$userid'";
	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

}

function updateUserLogin($site, $userid) {

	global $now;

	if (empty($now)) {
	    $now =time();
	}

	$userid=$this->preserveQuotes($userid);

	$result =  $this->query ("SELECT userid FROM ${site}_users WHERE userid='$userid'");
	if (!$result) { /* access problem */
		return -3;
	}

	if (!$this->next_record() ) {
		return ERR_NOTFOUND;
	}

	$result = $this->query ("UPDATE ${site}_users SET totallogins=totallogins+1, lastlogin='$now' WHERE userid='$userid'");
	if (!$result) {
		return -3;
	}
}


#	------------------------------------------------------------------
#				Permissions / User privileges
#	------------------------------------------------------------------

function getPrivs($site, $forum, $userid) {

	$userid=$this->preserveQuotes($userid);
	$result = $this->query ("SELECT * FROM ${site}_userforum WHERE bn_name='$forum' AND userid = '$userid'");
	if (!$result) {
		return -3;
	}
	
	if ($this->next_record() ) {
		return $this->Record;
	} else {
		return -3;
	}
}

function insertPrivs ($site, $forum, $userid, &$fields) {
	global $now;

	if (empty ($forum)) {
		$this->halt("insertPrivs(): invalid argument: forum not set");
		return -3;
	}

	if (empty ($userid)) {
		$this->halt("insertPrivs(): invalid argument: userid not set");
		return -3;
	}

	if (!is_array ($fields) ) {
		$this->halt("insertPrivs(): item to insert is not an array");
		return -3;
	}

# check if permissions already exists
	$this->query ("SELECT bn_name FROM ${site}_userforum WHERE bn_name='$forum' AND userid = '" . $this->preserveQuotes($userid) . "'");

	if ($this->next_record() ) {
		return $this->updatePrivs ($site, $forum, $userid, $fields);
	}

	if (empty($fields["unixdate"])) {
		$fields["unixdate"] = time();
	}
	
	$fields["userid"] = $userid;
	$fields["bn_name"] = $forum;

	return $this->insertRow("${site}_userforum", $fields);
}

function deletePrivs ($site, $forum="", $userid="") {


	$query = "DELETE FROM ${site}_userforum";
	if (!empty ($userid)) {
		$userid=$this->preserveQuotes($userid);
		$query .= " WHERE userid='$userid'";
	}
	
	if (!empty ($forum)) {
		if (empty ($userid)) {
			$query .= " WHERE bn_name='$forum'";
		} else {
			$query .= " AND bn_name='$forum'";
		}
	}

	$result = $this->query ($query);
	if (!$result) {
		return (-3);
	}
	return 0;
}

function updatePrivs ($site, $forum, $userid, $fields) {
	
	$sane_userid = $this->preserveQuotes($userid);
	$result = $this->query ("SELECT userid FROM ${site}_userforum WHERE userid='$sane_userid' AND bn_name='$forum'");
	if (!$this->next_record() ) {
		return -4; /* no row selected */
	}

	$fields["bn_name"] = $forum;
	$fields["userid"] = $userid;

	return $this->updateTable ("${site}_userforum", $fields, "userid='$sane_userid' AND bn_name='$forum'");
}

#	------------------------------------------------------------------
#				Forum/thread Mail subscriptions
#	------------------------------------------------------------------

/**
 * Subscribe a user to a forum / thread so that he receive new post by email
 * 
 * @param		string	$site			The site in which the forum is
 * @param		string	$forum			The forum where the user is subscribed to
 * @param		string	$useraddress	The email where to send new posts
 * @param		integer	$thread			thread number for which replies will be sent
 *										if 0 then all posts into the forum will be sent
 * @returns		integer	$result			0 if OK, negative value if error
 */
function subscribeUser ($site, $forum, $useraddress, $thread=0) {

	settype ($thread, "integer");

	$query = "SELECT thread FROM ${site}_userthread WHERE bn_name='$forum' AND useraddress='$useraddress' AND (thread=$thread";
	$query .= ($thread > 0) ? " OR thread=0)" : ")";

	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

# returns if user already subscribed
# ----------------------------------
	if ($this->next_record() ) {
		return 0;
	}

# If forum subscripton, remove all thread-user subscription records
# -----------------------------------------------------------------
	if ($thread == 0) {
		$query = "DELETE FROM ${site}_userthread WHERE useraddress='$useraddress' AND bn_name='$forum'";
		$result = $this->query ($query);
	}

# subscribes user to this thread
# ------------------------------

	$query = "INSERT INTO ${site}_userthread (bn_name, thread, useraddress) VALUES('$forum', $thread, '$useraddress')";
	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}
	return 0;
}

/**
 * Remove a user from a thread/forum subscription
 * 
 * @param		string	$site			The site in which the forum is
 * @param		string	$forum			The forum where the user is subscribed to, 
 *										if empty then the user will be removed from all forums in that site
 * @param		string	$useraddress	The email address to remove, if empty then all users will be removed
 * @param		integer	$thread			thread number for which to unsubcribe the user(s), 0 = all forum
 * @returns		integer	$result			0 if OK, negative value if error
 */
function unSubscribeUser ($site, $forum="", $useraddress="", $thread=0) {

	settype ($thread, "integer");

# Either forum, thread or useraddress must be set
# -----------------------------------------------
	if (empty($forum) && empty($thread) && empty($useraddress)) {
		return -3;
	}

	if ($thread > 0) {
		$query = "DELETE FROM ${site}_userthread WHERE thread=$thread";
	} else {
		$query = "DELETE FROM ${site}_userthread WHERE thread>=0";
	}

	if (!empty($forum) ) {
		$query .= " AND bn_name='$forum'";
	}
	
	if (!empty($useraddress)) {
		$query .= " AND useraddress='$useraddress'";
	}

	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}
}

/**
 * Check if a user has subscribed to a forum / thread
 * 
 * @param		string	$site			The site in which the forum is
 * @param		string	$forum			The forum to check subscription, 
 * @param		string	$useraddress	The email address to check
 * @param		integer	$thread			thread number or 0 = all forum
 * @returns		boolean	$subscribed			TRUE if user has subscribed
 */
function checkSubscribedUser ($site, $forum, $useraddress, $thread=0) {

	setType ($thread, "integer");

	$query = "SELECT useraddress FROM ${site}_userthread WHERE bn_name='$forum' AND thread=$thread AND useraddress='$useraddress'";
	$result = $this->query ($query);

	if ($result && $this->next_record() ) {
		return TRUE;
	}
	return FALSE;
}

/**
 * Get all user addresses that have subscribed to a forum / thread
 * 
 * @param		string	$site			The site in which the forum is
 * @param		string	$forum			The forum to check subscription, 
 * @param		integer	$thread			thread number or 0 = all forum
 * @returns		array	$users			The array of addresses [user@addr, user2@addr]
 */
function getSubscribedUsers ($site, $forum, $thread=0) {

	setType ($thread, "integer");

	$query = "SELECT DISTINCT useraddress FROM ${site}_userthread WHERE bn_name='$forum' AND (thread=$thread";
	$query .= ($thread > 0) ? " OR thread=0)" : ")";
	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

	$i=0;
	while ($this->next_record() ) {
		$useraddresses[$i++] = $this->f("useraddress");
	}
	return $useraddresses;
}

/**
 * Return the number of subscribed users in a site / forum
 * 
 * @param		string	$site			The site in which the forum is
 * @param		string	$forum			returns only subscriton count from forum '$forum' 
 * @returns		integer	$cnt			The number of subscribed users
 */
function countSubscribedUsers ($site, $forum='') {

	$query = "SELECT count(DISTINCT useraddress) AS cnt FROM ${site}_userthread WHERE thread=0";
	if ( !empty($forum) ) {
	    $query .= " AND bn_name='$forum'";
	}

	$result = $this->query ($query);
	if (!$result) {
		return -3;
	}

	if ($this->next_record() ) {
		return $this->Record['cnt'];
	}

	return 0;
}


#	------------------------------------------------------------------
#				Attachments management functions
#	------------------------------------------------------------------

/**
 * insert attachment reference in database 
 *
 * @param     string $note_id	id of the associated note
 * @param     string $name
 * @param     string $path
 * @param     string $inline	requested display mode
 * @param     string $size
 * @param     string $type
 * @param     string $state		state of attachement (1: active, 0, inactive, P: pending, D: Deleted)
 * @param     string $hidden
 * @access    public
 * @return    integer			The id of the attachment >0 if OK, a negative value in case of error
 */
function insertAttachment ($forum, $note_id, $name, $path, $inline="A", $size=0, $type="", $state='1', $hidden=0) {

	$upload_table = $this->site . "_attachments";

	$att_id = $this->nextId('attachments', $upload_table);
	if ($att_id <0) {
	    return -1;
	}

	$fields["att_id"]   = $att_id;
	$fields["unixdate"] = time();
	$fields["note_id"]  = $note_id;
	$fields["bn_name"]  = $forum;
	$fields["att_name"] = $name;
	$fields["att_path"] = $path;
	$fields["att_size"] = empty ($size) ? filesize($path) : $size;
	$fields["att_type"] = empty ($type) ? "application/octet-stream" : $type;
	$fields["state"]    = $state;
	$fields["hidden"]   = $hidden;
	$fields["inline"]   = $inline;
	$ret = $this->insertRow($upload_table, $fields);
	if ($ret != 0) {
		return -1;
	}

	return $this->lastId('attachments', $upload_table);
}

/**
 * Validate pending files, assign them to the new inserted note
 *	and remove all older files that have not been validated
 *
 * @param     string $tmp_id	temporary id that was previously assigned
 * @param     string $note_id	id of the associated note
 * @access    public
 * @return    boolean true if OK			
 */
function validateAttachments ($forum, $dir, $old_id, $new_id) {

	$upload_table = $this->site . "_attachments";
	$yesterday = time() - 86400;

	// validate 'pending' files
	$this->query ("UPDATE $upload_table SET note_id='$new_id', state='1' WHERE note_id='$old_id' AND state='P' AND bn_name='$forum'");

	// remove files elected to be deleted and purge old files (older than 1 day)
	$whereclause = "(bn_name='$forum') AND (note_id='$old_id' AND state='D') OR (state IN ('P', 'D') AND unixdate<$yesterday)";
	$this->query ("SELECT att_id, att_name FROM $upload_table WHERE $whereclause");
	$att_name = "";
	while ($this->next_record() ) {
		$att_name = $this-> f("att_name");
		$att_id = $this-> f("att_id");
		@unlink ("$dir/$att_id.$att_name");
	}

	if (!empty($att_name) ) {
		$this->query ("DELETE FROM $upload_table WHERE $whereclause");
	}

	// adjust attachments size
	$this->query ("SELECT sum(att_size) AS sz FROM $upload_table WHERE bn_name='$forum' AND note_id='$new_id' AND state='1'");
	if ($this->next_record()) {
		$att_size = $this-> f("sz");
		$this->query ("UPDATE $forum SET att_size='$att_size' WHERE cle='$new_id'");   
	}
	return true;
}

/**
 * Get attachment(s) associated to a note.
 *
 * Detail description
 * @param     
 * @since     1.0
 * @access    public
 * @return    array		attachments
 */
function getAttachments ($forum, $note_id, $att_id=0) {

	$upload_table = $this->site . "_attachments";

	$query = "SELECT att_id, att_name, att_type, att_size, att_path, inline, hidden, state FROM $upload_table WHERE note_id=$note_id AND bn_name='$forum' AND state IN ('P', '1') AND hidden!='1'";
	if ($att_id>0) {
	    $query .= " AND att_id=$att_id";
	}
	$this->query ($query);

	$i=0;
	while ($this->next_record() ) {
		$att[$i] = $this->Record;
		$i++;
	}

	return ($i==0 ) ? "" : $att;

}

/**
 * Flag the attachments as "deleted"
 *
 * @param	string $forum	The forum 
 * @param	string $key		the id of the associated note
 * @param	string $att_ids	ID(s) of attachments to be deleted (can be an array)
 * @since	4.1
 * @access	public
 * @return	void
 */
function setDeleteAttachment ($forum, $key, $att_ids) {

	$upload_table = $this->site . "_attachments";

	if (is_array($att_ids) ) {
		$att_ids = implode ($att_ids, ",");
		$q = "UPDATE $upload_table SET state='D' WHERE note_id='$key' AND bn_name='$forum' AND att_id IN ($att_ids)";
	} elseif (!empty($att_ids)) {
		$q = "UPDATE $upload_table SET state='D' WHERE note_id='$key' AND bn_name='$forum' AND att_id=$att_ids";
	} else {
	    return false;
	}
	
	$this->query($q);
}

/**
 * Flag the attachments as "inline"
 *
 * @param	string $forum	The forum 
 * @param	string $key	the id of the associated note
 * @param	string $att_id	ID of attachment to be set to inline
 * @param	string $mode	inline mode (1 = Inline, 0 = as Link)
 * @access	public
 * @return	void
 */
function setInlineAttachment ($forum, $key, $att_id, $mode=0) {

	$upload_table = $this->site . "_attachments";

	$q = "UPDATE $upload_table SET inline='$mode' WHERE note_id='$key' AND bn_name='$forum' AND att_id=$att_id";
	$this->query($q);
}

/**
 * Delete all attachments associated with a note
 *
 * @param	string $forum	The forum 
 * @param	string $key		the id of the associated note
 * @param	string $dir		The directory that contain attachments
 * @param	string $att_id	ID of attachment to be deleted, if none, all will be deleted
 * @since	4.1
 * @access	public
 * @return	void
 */
function deleteAttachment ($forum, $key, $dir, $att_id=0) {

	$upload_table = $this->site . "_attachments";

	$whereclause = "bn_name='$forum' AND note_id=$key";
	if ($att_id>0) {
	    $whereclause .= " AND att_id=$att_id";
	}
	$this->query ("SELECT att_id, att_name FROM $upload_table WHERE $whereclause");

	while ($this->next_record() ) {
		$att_name = $this-> f("att_name");
		$att_id = $this-> f("att_id");
		@unlink ("$dir/$att_id.$att_name");
	}

	$this->query ("DELETE FROM $upload_table WHERE $whereclause");
}


#	------------------------------------------------------------------
#				Notes management functions
#	------------------------------------------------------------------

function getNote ($forum, $key, $where="") {

	$site = $this->site;

	$this->query ("SELECT newest FROM $forum WHERE cle=$key");
	if ($this->next_record() ) {
		$newest = (int) $this->Record["newest"];
	} else {
		return -1;
	}

# get key of previous thread
# --------------------------
	$whereclause = "hidden=0 AND newest > " . $newest;
	if (!empty($where)) {
		$whereclause .= " AND $where";
	}

	$this->query ("SELECT min(newest) AS prev FROM $forum WHERE $whereclause");
	if ($this->next_record() ) {
		$prev = (int) $this->Record["prev"];
		if ($prev) {
			$this->query ("SELECT thread AS tprev FROM $forum WHERE newest=$prev");
			if ($this->next_record() ) $tprev = $this->Record["tprev"];
		}
	}

# get key of next thread
# ----------------------
	$whereclause = "hidden=0 AND newest < " . $newest;
	if (!empty($where)) {
		$whereclause .= " AND $where";
	}

	$this->query ("SELECT max(newest) AS next FROM $forum WHERE $whereclause");
	if ($this->next_record() ) {
		$next = (int) $this->Record["next"];
		if ($next) {
			$this->query ("SELECT thread AS tnext FROM $forum WHERE newest=$next");
			if ($this->next_record() ) $tnext = $this->Record["tnext"];
		}
	}

# get all fields in selected note
# -------------------------------
	$result = $this->query ("SELECT * FROM $forum WHERE cle=$key");
	if (! $this->next_record() ) {
		return -4;
	}

	$note = $this->Record;
	$note["tprev"] = $tprev;
	$note["tnext"] = $tnext;

	// Get user info from users table
	$uid = $note['userid'];
	if (!empty($uid)) {    
		$result = $this->query ('SELECT * FROM '.$this->site.'_users WHERE userid =\''.$this->preserveQuotes($uid)."'");
		if ($this->next_record() ) {
			$this->users[$uid] = $this->Record;
			reset($this->Record);
			while ( list($var, $val) = each($this->Record) ){
				$n = 'user_' . $var;
				$note[$n] = $val;
			}
		}
	}
	return $note;
}

/**
 * Get latest updated thread
 */
function getNewestThread ($forum, $where="") {

	$whereclause = "hidden=0";
	if (!empty($where)) {
		$whereclause .= " AND $where";
	}

	$this->query ("SELECT max(cle) AS latest FROM $forum WHERE $whereclause");
	if (!$this->next_record() ) {
		return -4;
	}

	$cle = $this->Record["latest"];
	$this->query ("SELECT thread FROM $forum WHERE cle=$cle");
	if (!$this->next_record() ) {
		return -4;
	}

	return $this->Record["thread"];
}

function getThread ($forum, $key, $thread) {

	unset ($this->entries);
	unset ($this->children);

	$whereclause = "hidden=0 AND thread=$thread";

	reset ($GLOBALS["list_var"]);
	$cols = "cle,parent,childs,thread,newest";
	while (list($var,$val) = each($GLOBALS["list_var"])) {
		if ($val==1) $cols.=",$var";
	}

	$result = $this->query ("SELECT $cols FROM $forum WHERE $whereclause ORDER BY unixdate");
	if (!$result) {
		return (-3);
	}

	$count = 0;
	$set = '';
	while ($this->next_record() ) {
		$key = (int) $this->f("cle");
		$parent = (int) $this->f("parent");
		$this->entries[$key] = $this->Record;
		$this->children[$parent][] = $key;
		$count++;

		if ( isset($this->Record['userid'])  )  {
			$uid = $this->Record['userid'];
			if (!isset($this->users[$uid])) {
				$this->users[$uid] = array();
				$set .= "'" .$this->preserveQuotes($uid)."',";
			}
		}
	}

	if (!empty($set)) {
		$result = $this->query ('SELECT * FROM '.$this->site.'_users WHERE userid in ('.ereg_replace (',$', '', $set).')');
		while ($this->next_record() ) {
			$uid = $this->Record['userid'];
			$this->users[$uid] = $this->Record;	    
		}
	}
	    
	return $count;
}

function getChild ($key, $action) {

	global $now;

	if (!is_array($this->entries)) return -1;
	reset($this->entries);
	$this->lineNo = 0;

	$GLOBALS["bn_${action}_begin_level"] = str_replace('"','\"', $GLOBALS["bn_${action}_begin_level"])."\n";
	$GLOBALS["bn_${action}_odd_first_line"] = str_replace('"','\"', $GLOBALS["bn_${action}_odd_first_line"])."\n";
	$GLOBALS["bn_${action}_even_first_line"] = str_replace('"','\"', $GLOBALS["bn_${action}_even_first_line"])."\n";
	$GLOBALS["bn_${action}_odd_line"] = str_replace('"','\"', $GLOBALS["bn_${action}_odd_line"])."\n";
	$GLOBALS["bn_${action}_even_line"] = str_replace('"','\"', $GLOBALS["bn_${action}_even_line"])."\n";
	$GLOBALS["bn_${action}_end_level"] = str_replace('"','\"', $GLOBALS["bn_${action}_end_level"])."\n";

	$this->displayChild ($key, $action);
}

function displayChild ($key, $action) {
	global $now, $main_page, $ext, $site, $bn, $bn_db, $bn_action, $bn_pad, $bn_depth_limit;
	global $cookies, $auth, $db;
	global $note, $lynx, $pattern;
	global $icon_dir, $img_dir, $bn_dir_notes;

	global $msg;
	global $expnd, $here;
	global $first, $last, $next, $prev;
	global $total, $totalthreads, $totalnotes, $notes, $threads;
	global $forum_types, $forum_states;

	global $bn_win_width, $bn_win_height;
	global $list_url, $list_text, $list_link;
	global $post_url, $post_text, $post_link;
	global $search_url, $search_text, $search_link;
	global $show_profile_url, $show_profile_text, $show_profile_link;
	global $list_prev_url, $list_prev_text, $list_prev_link;
	global $list_next_url, $list_next_text, $list_next_link;
	global $view_prev_url, $view_prev_text, $view_prev_link;
	global $view_next_url, $view_next_text, $view_next_link;
	global $user_priv;

	static $threadNo = 0;
	static $level = 0;

	$top_level=0;

	$has_children=is_array($this->children[$key]);
	$is_expanded=1;
	setType ($bn_depth_limit, "integer");
	// evaluate all variables
	if (($key>0) && is_array($this->entries[$key]) ) {
		reset($this->entries[$key]);
		while ( list( $var, $val ) = each( $this->entries[$key])) {
			if (is_string ($var)) {
				// global $$var;
				$$var = $val;
				if ($var=="body") {
					$body=format_text($body);
				}
			}
		}
		if (isset($this->users[$userid])) {
		    extract ($this->users[$userid], EXTR_PREFIX_ALL, 'user');
		}
		// USER PRIVILEGES
		if (isset($user_userpriv) ) {
			$user_priv = $GLOBALS['user_privs'][$user_userpriv];    
		}

	}

# Handle cookies / new stuff
# --------------------------

	$new_icon = "";
	$is_new = FALSE;
	$is_updated = FALSE;
	$LVisit = 0;
	if ($GLOBALS["bn_cookies"] && isSet ($cookies)) {
		// Check if session cookie set
		if (isSet ($cookies->s_cookies["LastVisit"])) {
			$LVisit=$cookies->s_cookies["LastVisit"];
			if ( ($unixdate > $LVisit) && !ereg($key, $cookies->s_cookies["visited"]) ) {
				$is_new = TRUE;
				$new_icon=msg(82);
			} elseif ( ($newest > $LVisit) && !ereg($key, $cookies->s_cookies["visited"]) ) {
				$is_updated = TRUE;
			}
		} else {
			// No cookie set, disable the feature 
			$LVisit=0;
			$is_new = TRUE;
		}
	}

# set color depending on note age
# -------------------------------
	$age = $GLOBALS[now] - $unixdate;
	if ($age > ( (int) $GLOBALS['bn_new_age']) ) {
		$date_color = $GLOBALS['bn_normal_color'];
	} elseif ($age < ( (int) $GLOBALS['bn_hot_age']) ) {
		$date_color = $GLOBALS['bn_hot_color'];
	} else {
		$date_color = $GLOBALS['bn_new_color'];
	}

# don't display first "pseudo" message => only contains children
# --------------------------------------------------------------
	$hidden_note = ($key>0) ? false : true;

# shows Hidden notes only to moderator
# ------------------------------------
	if ( ($hidden && $auth->level<1) ) {
		$hidden_note = true;
	}

# don't display old notes if "list only new" is set
# -----------------------------------------------
	if ( ($expnd == "new") && !$is_new ) {
		$hidden_note = true;
	}

	if (!$hidden_note)  {
		$formated_date = date ($GLOBALS["bn_date_format"], $unixdate+($GLOBALS["bn_tz_offset"]*3600));
		$note = $key;

		if ($att_size>0) {    
			$att = $this->getAttachments ($bn_db, $key);
			if (is_array($att)) {
				$att_count = count($att);
				for ($i=0; $i<$att_count; $i++) {
					$att_id        = $att[$i]["att_id"];
					$att_ids[$i]   = $att_id;
					$att_name      = $att[$i]["att_name"];
					$att_names[$i] = $att_name;
					$att_type      = $att[$i]["att_type"];
					$att_types[$i] = $att_type;
					$att_size      = (integer) $att[$i]["att_size"];
					$att_sizes[$i] = $att_size;
					$att_url       = "getfile.$ext?bn=$bn&key=$key&att_id=$att_id";
					$att_urls[$i]  = $att_url;
					$att_inline    =  $att[$i]["inline"];
					$att_link     = getAttachmentUrl ($key, $att_id, "$bn_dir_notes/$att_id.$att_name", $att_type, $att_size, $att_inline);
					$att_links[$i]  = $att_link;
					$attachments .= $att_links[$i]."&nbsp;";
					eval ("\$item = (empty(\$msg[193])) ? \"\$att_name\" : \"$msg[193]\";");
					$att_list_text .= "\n$item";
					$att_list["$att_id"] = $item;
				}
				$msg74 = str_replace('"','\"', $msg[74]);
				eval("\$att_icon = \"$msg74\";");
				$att_list_box = getListBox ("att_id", $att_list, "", 0, "window.location=('getfile.$ext?site=$site&bn=$bn&key=$key&att_id='+this.options[this.selectedIndex].value)");
				$att_list_form = "<form action=\"getfile.$ext\"><input type=\"hidden\" name=\"bn\" value=\"$bn\"><input type=\"hidden\" name=\"key\" value=\"$key\">".msg(57).$att_list_box."<input type=\"submit\" value=\"".msg(194,"download")."\"></form>";
				unset ($att);
			}
		}

		// determine if top level of thread (new thread)
		if ($parent==0) {
			$top_level=1;
			$threadNo++;
			$threadAnchor="<A NAME=\"A$thread\"></A>";
		}
		else {
			$top_level=0;
			$threadAnchor="";
		}

		// initialize "latest" pattern
		if (empty($newest)) {
			$newestNote="";
			$newestPrmpt="";
		} else {
			$newestNote= date ($GLOBALS["bn_date_format"], $newest+($GLOBALS["bn_tz_offset"]*3600));
			$m=str_replace('"','\"', $msg[68]);
			eval ("\$newestPrmpt=stripslashes(\"$m\");");
		}

		// set the pointer to the current note 
		if (isset($here) && ($here == $key)) {
			$currPointer=msg(73);
		} else {
			$currPointer="";
		}

		// is this entry at top of thread (ie. new topic)
		if ($top_level) {

			if ($closed == 1) {
				$thread_icon = msg(146, "<img src='images/locked_thread.gif'>");	// closed thread
			} elseif ($is_updated || $is_new) {			
				if ($replies > $GLOBALS["bn_hot_threshold"]) {
					$thread_icon = msg (147, "<img src='images/hot_new_thread.gif'>");	// thread with new messages and more than XX replies
				} else {
					$thread_icon = msg (148, "<img src='images/new_thread.gif'>");	// thread with new messages in it
				}
			} else {
				if ($replies > $GLOBALS["bn_hot_threshold"]) {
					$thread_icon = msg (149, "<img src='images/hot_thread.gif'>");	// thread with more than XX replies
				} else {
					$thread_icon = msg (150, "<img src='images/thread.gif'>");	// normal (old) thread
				}
			}

			if ($replies > 0) {
				// if newest note is since last visit, force thread to be expanded
				if ( ($expnd!="none") && $GLOBALS["bn_cookies"] && isSet ($cookies) && ($newest > $LVisit) && !ereg($newest, $cookies->s_cookies["visited"]) ) {
					$is_expanded = 1;
				} else if ($expnd==$key || $expnd=="all") {
					$is_expanded = 1;
				} else {
					$is_expanded = 0;
				}

				// set expand_icon depending on collapsed or expanded:
				$expand_icon="<a href='list.$ext?bn=$bn";
				if ($first) $expand_icon .= "&first=$first";
				if ($last) $expand_icon .= "&last=$last";
				if ($is_expanded) {
					// currently expanded
					$expand_icon .= "#A$thread'>".msg(75)."</a>";
				} else {
					// currently collapsed
					$expand_icon .= "&expnd=$thread#A$thread'>" . msg(76) . "</a>";
				}
			} else {			
				$expand_icon = msg(72); // set expand_icon to "single message"
			}
			$line = ($this->lineNo % 2) ? "bn_${action}_odd_first_line" : "bn_${action}_even_first_line";
		} else {
			$expand_icon = "";
			$line = ($this->lineNo % 2) ? "bn_${action}_odd_line" : "bn_${action}_even_line";
			$replies=0;
			if (!empty($bn_pad)) {
				if (empty($bn_depth_limit) ) {
					$depth = $level;
				} else {
					$depth = ($level<$bn_depth_limit) ? $level : $bn_depth_limit;
				}
				$pad = "";
				while ($depth--) {
					$pad .= "$bn_pad";
				}
			}
		}

		// set URL to display that note
		$view_url ="view.$ext?site=$site&bn=$bn&key=$key";
		$edit_url ="edit.$ext?site=$site&bn=$bn&key=$key";
		$reply_url ="post.$ext?site=$site&bn=$bn&key=$key";
		$subscribe_thread_url = "subscribe_thread.$ext?site=$site&bn=$bn&thread=$thread";
		$close_url = "close.$ext?bn=$bn&key=$thread";
		if (!empty($first)) {
			$view_url .= "&first=$first";
			$edit_url .= "&first=$first";
			$reply_url .= "&first=$first";
		}
		if (!empty($last)) {
			$view_url .= "&last=$last";
			$edit_url .= "&last=$last";
			$reply_url .= "&last=$last";
		}
		if (!empty($pattern)) {
			$view_url .= "&pattern=$pattern";
		}
		$view_link = "<a href=\"$view_url\">$subject</a>";

		$reply_link="";
		if ($GLOBALS["bn_thread"]) {
			$reply_text = msg(50, LABEL_REPLY_MESSAGE);
			$reply_link = anchorWindow ($reply_url, $reply_text, msg (165, ALT_REPLY_MESSAGE), $bn_win_width, $bn_win_height, $link_class);
		}

		$edit_link="";
		if ($GLOBALS["bn_modify"]) {
			$edit_text = msg(60, LABEL_EDIT_MESSAGE);
			$edit_link = anchorWindow ($edit_url, $edit_text, msg (166, ALT_EDIT_MESSAGE), $bn_win_width, $bn_win_height, $link_class);
		}

		$delete_link = "";
		if ($GLOBALS["bn_delete"]) {
			$delete_url ="delete.$ext?site=$site&bn=$bn&key=$key";
			$delete_text = msg(42, LABEL_DELETE_MESSAGE);
			$delete_link = anchor ($delete_url, "", $delete_text, msg (167, ALT_DELETE_MESSAGE), $link_class);
		}

		// SHOW PROFILE
		$show_profile_url = "";
		if ($userid != "guest") {
			$show_profile_url = "profile.$ext?bn=$bn&showuser=$userid";
			$show_profile_text = msg(187, "<img src=\"$icon_dir/profile.gif\" width=15 height=15 border=0>");
			$show_profile_link = anchor ($show_profile_url, "", $show_profile_text, sprintf (ALT_SHOW_PROFILE, $userid), $link_class );
		}

		// display line
		if (!empty($GLOBALS[$line]) ) {
			$this->lineNo ++;

			if (!empty($GLOBALS["bn_before_line"]) ) {
				eval ($GLOBALS["bn_before_line"]);
			}

			$l = $GLOBALS[$line];
			eval ("echo stripSlashes(\"$l\");");
		}
	}

	if ($has_children && $is_expanded) {
		// display begin level pattern
		if ($key>0) {
			$level++;
			if ( ($level<$bn_depth_limit)||($bn_depth_limit==0) ) {
				$l = $GLOBALS["bn_${action}_begin_level"];
				eval ("echo stripSlashes(\"$l\");");
			}
		}
		// display all replies to this thread
		for (reset ($this->children[$key]); $child=current($this->children[$key]); next($this->children[$key])) {
			$this->displayChild ($child, $action);
		}

		// display end of level pattern
		if ($key>0) {
			if ( ($level<$bn_depth_limit)||($bn_depth_limit==0) ) {
				$l = $GLOBALS["bn_${action}_end_level"];
				eval ("echo stripSlashes(\"$l\");");
			}
			$level--;
		}
	}
	
	// display end of thread/level pattern
	if ($top_level) {
		$l=str_replace('"','\"', $GLOBALS["bn_${action}_end_thread"]);
		eval ("echo stripSlashes(\"$l\");");
	}
}

function listThreads ($forum, $first=0, $last=0, $where="", $expanded=1, $limit) {
	 echo "function <b>listThreads()</b> not implemented<br>";
}

function listNotes ($forum, $first=0, $last=0, $sort="", $where="", $limit) {
	 echo "function <b>listNotes()</b> not implemented<br>";
}

function insertNote ($forum, $key, &$fields) {

# check if note already exists
# ----------------------------
	$result = $this->query ("SELECT cle FROM $forum WHERE cle=$key");
	if (!$result) {
		return -3;
	}

	if ($this->next_record() ) {
		return ERR_EXISTS;
	}

#	Insert note
#	-----------
	$fields["parent"] = 0;
	$fields["thread"] = $key;
	$fields["childs"] = 0;
	if (empty($fields["unixdate"])) {
		$fields["unixdate"] = time();
	}
	if (empty($fields["mod_date"])) {
		$fields["mod_date"] = $fields["unixdate"];
	}
	$fields["newest"] = $fields["unixdate"];
	$fields["cle"] = $key;
	$rc = $this->insertRow($forum, $fields);

#	Update forum statistics
# -----------------------
	if ($rc == 0) {
		$this->updateForumStats ($this->site, $forum);
		return $key;
	} else {
		return $rc;
	}
}

function threadNote ($forum, $key, $parent_key, &$fields) {

# 	Check if parent exists and get 'thread id'
#	------------------------------------------
	$result = $this->query ("SELECT thread FROM $forum WHERE cle=$parent_key");
	if (!$this->next_record() ) {
		return -4;
	}

	$thread = $this->f("thread");

# check if note already exists
# ----------------------------
	$result = $this->query ("SELECT cle FROM $forum WHERE cle=$key");
	if ($this->next_record() ) {
		return ERR_EXISTS;
	}

#	Insert note
#	-----------
	$fields["parent"] = $parent_key;
	$fields["thread"] = $thread;
	$fields["childs"] = 0;

	if (empty($fields["unixdate"])) {
		$fields["unixdate"] = time();
	}

	if (empty($fields["mod_date"])) {
		$fields["mod_date"] = $fields["unixdate"];
	}

	if (empty($fields["hidden"])) {
		$fields["newest"] = $fields["unixdate"];
	}
	$fields["cle"] = $key;

	$rc = $this->insertRow ($forum, $fields);
	if ($rc != 0) {
		return $rc;
	}

#	Update parent note
#	------------------
	$result = $this->query ("UPDATE $forum SET childs=childs+1 WHERE cle=$parent_key");
	if (!$result) {
		return -3;
	}

#	Update "newest" field in all notes in this thread
#	-------------------------------------------------
	if (empty($fields["hidden"])) {
		$this->updateNewest ($forum, $thread);
	}

#	Update forum statistics
#	-----------------------
	$this->updateForumStats ($this->site, $forum);
	return $key;
}

function updateNote ($forum, $key, &$fields) {

# get the current thread id and last in thread
# --------------------------------------------
	$result =  $this->query ("SELECT thread, newest FROM $forum WHERE cle=$key");
	if (!$this->next_record() ) {
		return -4; /* no row selected */
	}

	$thread = (int) $this->f("thread");
	$newest = (int) $this->f("newest");

	if (empty($fields["mod_date"])) {
		$fields["mod_date"] = $GLOBALS["now"];
	}

# Update note in database
# -----------------------
	$result = $this->updateTable ($forum, $fields, "cle=$key");
	if ($result != 0) {
		return $result;
	}

# Update newest in thread if timestamp updated
# --------------------------------------------
	if ($fields["unixdate"] > $newest) {
		$this->updateNewest ($forum, $thread, $fields["unixdate"]);
	}

	return 0;
}

/**
 * Update recursively the 'thread' in all descendants of a note
 *
 * @access	private
 * @param	string	$forum		the forum (tablename) in which the thread is located
 * @param	integer	$parent		note id for which all descendants will be updated
 * @param	integer	$newthread	the new 'thread' value
 */
function updateThread ($forum, $parent, $newthread) {

# Update thread value for all immediate childs of 'parent'
# --------------------------------------------------------
	$result = $this->query ("UPDATE $forum SET thread=$newthread WHERE parent=$parent");

# Gets all childrens and sub-childs count
# ---------------------------------------
	$result = $this->query ("SELECT cle, childs FROM $forum WHERE parent=$parent");
	while ($this->next_record() ) {
		$cle = (int) $this->f("cle");
		$childkeys[$cle] = (int) $this->f("childs");
	}
		
# if immediate children have children => update the 'thread' column recursively
# -----------------------------------------------------------------------------
	if (is_array($childkeys) ) {
		reset ($childkeys);
		while ( list ($key, $childs) = each ($childkeys) ) {
			if ($childs > 0) {
				$this->updateThread ($forum, $key, $newthread);
			}
		}
	}
}

/**
 * Update the 'newest' column for all notes in a thread
 *
 * @access	private
 * @param	string	$forum		the forum (tablename) in which the thread is located
 * @param	integer	$thread		The thread to be updated
 * @param	integer	$newest		the new 'newest' value, if empty then gets the newest from the thread
 */
function updateNewest ($forum, $thread, $newest="") {
	if ($newest=="") {
		$this->query ("SELECT MAX(unixdate) AS newest FROM $forum WHERE thread=$thread AND hidden=0");
		if ($this->next_record() ) {
			$newest = $this->f("newest");
		}
	}
	if (!empty($newest) ) {
		$this->query ("UPDATE $forum SET newest=$newest WHERE thread=$thread");
	}
}

function deleteNote ($forum, $key, $bn_dir_notes) {

	$site = $this->site;

	$result =  $this->query ("SELECT filename, parent, childs, thread FROM $forum WHERE cle=$key");
	if (!$result) { /* access problem */
		return -3;
	}

	if (!$this->next_record() ) {
		return -4; /* no row selected */
	}

	$filename = $this->f("filename");
	$parent = (int) $this->f("parent");
	$childs = (int) $this->f("childs");
	$thread = (int) $this->f("thread");

	// delete note
	$this->query("DELETE FROM $forum WHERE cle=$key");

	if ($parent > 0) {
		// decrement childs count in parent
		$this->query ("UPDATE $forum SET childs=childs-1 WHERE cle=$parent");
		$this->updateNewest ($forum, $thread);
	} else {
		// This note was at root thread level => all children become new threads
		$this->query ("SELECT cle FROM $forum WHERE parent=$key");

		while ($this->next_record() ) {
			$childkeys[] = (int) $this->f("cle");
		}
		
		if (is_array($childkeys) ) {
			for (reset ($childkeys); $child=current($childkeys); next($childkeys)) {
				// Update thread value for all immediate childs of 'parent'
				$this->query ("UPDATE $forum SET thread=$child WHERE cle=$child");
				$this->updateThread ($forum, $child, $child);
				$this->updateNewest ($forum, $child);
			}
		}
		// remove all subscribed users to this thread
		$this->unSubscribeUser ($this->site, $forum, "", $key);
	}
	
# Link children to parent
# -----------------------
	if ($childs > 0) {
		$this->query ("UPDATE $forum SET parent=$parent WHERE parent=$key");
	}

# remove HTML file (static mode)
# ------------------------------
	if (!empty($filename) && ($filename != "none") && is_file("$bn_dir_notes/$filename")) {
 		@unlink ("$bn_dir_notes/$filename");
	}

# Remove attachment(s)
# --------------------
	$this->deleteAttachment ($forum, $key, $bn_dir_notes);

# Update statistics
# -----------------
	$this->updateForumStats ($this->site, $forum);

	return 0;
}

function mDeleteNotes ($forum, $notes, $bn_dir_notes) {

	for (reset ($notes); $note=current($notes); next($notes)) {
		$ret = $this->deleteNote($forum, $note, $bn_dir_notes);
		if ($ret < 0)
			return $ret;
		else
			$i++;
	}
	return 0;
}

function mDeleteThreads ($forum, $notes, $bn_dir_notes) {

	for (reset ($notes); $note=current($notes); next($notes)) {
		$ret = $this->deleteThread($forum, $note, $bn_dir_notes);
		if ($ret < 0) {
			return $ret;
		}
	}
	return 0;
}

function deleteThread ($forum, $thread, $bn_dir_notes) {

# Remove static HTML files + get all keys
# ---------------------------------------
	$result =  $this->query ("SELECT cle, filename FROM $forum WHERE thread=$thread");
	while ($this->next_record() ) {
		$filename = $this-> f("filename");
		$keys[] = $this-> f("cle");

		// remove HTML file (static mode)
		if (!empty($filename) && ($filename != "none") && is_file("$bn_dir_notes/$filename")) {
			@unlink ("$bn_dir_notes/$filename");
		}
	}

# remove attachments
# ------------------
	reset ($keys);
	while ( list(,$key) = each ($keys) ) {
		$this->deleteAttachment ($forum, $key, $bn_dir_notes);
	}

# delete thread
# -------------
	$this->query("DELETE FROM $forum WHERE thread=$thread");

# remove all subscribed users to this thread
# ------------------------------------------
	$this->unSubscribeUser ($this->site, $forum, "", $thread);
	
# Update forum statistics
# -----------------------
	$this->updateForumStats ($this->site, $forum);

	return 0;
}

function copyThread ($forum1, $thread1, $bn_dir_notes1, $forum2, $thread2=0, $bn_dir_notes2) {

# Get all notes in old thread
# ---------------------------
	$result =  $this->query ("SELECT * FROM $forum1 WHERE thread=$thread1 ORDER BY unixdate");
	while ($this->next_record() ) {
		$key = $this-> f("cle");
		$old_thread[$key] = $this->Record;
	}

# Copy each notes to new thread
# ------------------------
	reset ($old_thread);
	while ( list($oldkey, $note) = each ($old_thread) ) {
		$parent = $note['parent'];
		$trykey = $oldkey;
		if ($parent==0) {
			do {
				$newkey = $this->insertNote ($forum2, $trykey, $note);
			    $trykey++;
			} while ($newkey==ERR_EXISTS);
		} elseif (isset($newkeys[$parent]) ) {
			do {
				$newkey = $this->threadNote ($forum2, $trykey, $newkeys[$parent], $note);
			    $trykey++;
			} while ($newkey==ERR_EXISTS);
		} else {
			// should not occur
			$this->halt (ERROR_PARENT_NOT_FOUND);
		    return -1;
		}

		if ($newkey>0) {
			$newkeys[$oldkey] = $newkey;
		} else {
		    return $newkey;
		}

		// copy all attachments
		$att = $this->getAttachments ($forum1, $oldkey);
		if (is_array($att)) {
			$att_count = count($att);
			for ($i=0; $i<$att_count; $i++) {
				$oldid  = $att[$i]["att_id"];
				$name   = $att[$i]["att_name"];
				$type   = $att[$i]["att_type"];
				$inline =  $att[$i]["inline"];
				$state  =  $att[$i]["state"];
				$hidden =  $att[$i]["hidden"];
				$size   = (integer) $att[$i]["att_size"];
				$id=$this->insertAttachment ($forum2, $newkey, $name, $bn_dir_notes2, $inline, $size, $type, $state, $hidden);
				if ($id>0) {
					copy ("$bn_dir_notes1/$oldid.$name", "$bn_dir_notes2/$id.$name");
				} else {
					$this->halt (str_replace("{NAME}",$name, str_replace("{FORUM}",$forum2, str_replace("{CLE}", $newkey, ERROR_INSERT_ATTTACHMENT))));
				    return $id;
				}
			}
		}

	}

# Update forum statistics
# -----------------------
	$this->updateForumStats ($this->site, $forum2);

	return 0;
}


/**
 *	Get all threads (threads_id) opened or modified betwwen 2 given dates
 *
 *	@param	string	$forum		Table name which contains the note to be deleted
 *	@param	integer	$from		first date (unix timestamp format)
 *	@param	integer	$to			last date (unix timestamp format)
 *	@param	string	$type		"lastdate" = last post date in thread or "startdate" = start date of threads
 *	@return	array	$threads	An array with values set to the thread id 
 */

function getThreadsDate ($forum, $from, $to, $type="lastdate") {

	$col = ($type=="lastdate") ? "newest" : "thread";

	$result = $this->query ("SELECT DISTINCT thread FROM $forum WHERE $col>=$from AND $col <= $to");
	if (!$result) {
		return 0;
	}

	while ($this->next_record() ) {
		$threads[] = $this-> f("thread");
	}
	return $threads;
}

/**
 *	delete entire threads by given date
 *
 *	@param	string	$forum			Table name which contains the note to be deleted
 *	@param	integer	$from			first date (unix timestamp format)
 *	@param	integer	$to				last date (unix timestamp format)
 *	@param	string	$bn_dir_notes	directory in which the static HTML files and attachments are stored
 *	@param	string	$type			"lastdate" = last post date in thread or "startdate" = start date of threads
 */
function mDeleteThreadsDate ($forum, $from, $to, $bn_dir_notes, $type="lastdate") {

	$col = ($type=="lastdate") ? "newest" : "thread";

# delete static HTML files and attachments and unsubscribe users
# --------------------------------------------------------------
	$result = $this->query ("SELECT cle, thread, filename FROM $forum WHERE $col>=$from AND $col <= $to");
	if (!$result) {
		return 0;
	}

	while ($this->next_record() ) {
		$cle = $this-> f("cle");
		$thread = $this-> f("thread");
		$filename = $this-> f("filename");
		$keys[] = $this-> f("cle");

		// remove HTML file (static mode)
		if (!empty($filename) && ($filename != "none") && is_file("$bn_dir_notes/$filename")) {
			@unlink ("$bn_dir_notes/$filename");
		}

		// remove all subscribed users to this thread
		if ($cle == $thread) {	// if this is the first message in the thread
			$this->unSubscribeUser ($this->site, $forum, "", $thread);
		}

	}

# Remove attachments
# ------------------
	if (is_array($keys)) {
		reset ($keys);
		while ( list(,$key) = each ($keys) ) {
			$this->deleteAttachment ($forum, $key, $bn_dir_notes);
		}
	}

# delete threads
# --------------
	$this->query("DELETE FROM $forum WHERE $col >= $from AND $col <= $to");

# Update forum statistics
# -----------------------
	$this->updateForumStats ($this->site, $forum);
}

function mDeleteDate ($forum, $from, $to, $bn_dir_notes) {

# Gets all individual notes in the range [$from - $to]
# ----------------------------------------------------
	$result = $this->query ("SELECT cle FROM $forum WHERE cle>=$from AND cle <= $to");
	if (!$result) {
		return 0;
	}

# stores all keys in an array (cannot perform embedded SQL queries w/ PHPLIB)
# ---------------------------------------------------------------------------
	while ($this->next_record() ) {
		$notes[] = $this-> f("cle");
	}

	return $this->mDeleteNotes ($forum, $notes, $bn_dir_notes);
}

function setClosedThread ($forum, $thread, $closed=1) {

	settype ($closed, "integer");

	// check that thread exists
	$result =  $this->query ("SELECT cle FROM $forum WHERE cle=$thread");

	if (!$result) {
		return -3;	/* access problem */
	}

	if (!$this->next_record() ) {
		return -4;	/* no row selected */
	}

	// (un)close Thread
	$this->query("UPDATE $forum SET closed=$closed WHERE thread=$thread");
}

/**
 * Build the where clause used in the search function
 *
 * @param	string	$pattern		The pattern to search
 * @param	array	$search_fields	An array that contain the fields name to be searched
 * @param	integer	$search_type	search mode : 0= all words, 1=any word, 2=exact phrase
 * @param	integer	$casesensitive	Case sensitive search (1 = yes, default: 0 = no)
 * @param	string	$where			An optional where clause
 *
 * @access  public
 * @return	string	the where clause
 */
function buildSearchWhereClause ($pattern, &$search_fields, $search_type, $casesensitive=0, $where="") {

 # build the search terms arrays
 # -----------------------------
	if ($search_type == 2) {
		// search exact phrase
		$terms[]=$pattern;
	} else {
		$terms = split_search_terms ($pattern);
	}

 # build the query
 # ---------------
	$whereclause = "hidden=0";

	// for each term to search...
	if (is_array($terms) ) {
		reset ($terms);
		while (list (, $term) = each ($terms)) {
			if (!empty($term) ) {
				$term = $this->preserveQuotes($term);
				if(substr($term, 0, 1) == "-") {
					$term =substr($term, 1);
					$like = "NOT LIKE";
				} else {
					$like = "LIKE";
				}
				// ... select the columns to look in
				if ($casesensitive) {
					$searchterms[] = '( ' . implode ($search_fields, " $like '%$term%' OR ") . " $like '%$term%' )";
				} else {
					$searchterms[] = '( upper(' . implode ($search_fields, ") $like upper('%$term%') OR upper(") . ") $like upper('%$term%') )";
				}
			}
		}
	}

	// now build the complete query, merging all partial term search clauses
	if (is_array ($searchterms) ) {
		$cmp = ($search_type==0) ? "AND" : "OR";
		$whereclause .= " AND (" . implode (" $cmp ", $searchterms) . ") ";
	}

	if (!empty($where) ) {
		$whereclause .= " AND ($where)";
	}
	return $whereclause;

} // end func

} // end Class

} // defined DB_ACCESS
?>
