<?php

/*
 *  W-AGORA 4.0
 *  -----------
 *  $Id: postgres65.php,v 1.18 2002/06/10 17:41:35 mdruilhe Exp $
 *  Usage:              include file - Database access functions (PostgreSQL >= v6.5)
 *  Author:             Marc Druilhe <mdruilhe@w-agora.com>
 *  Initial file by:    Ron Friedman <friedman@filmmaker.com>
 */

if (!defined('_GLOBALS')) {
	die('Hacking attempt');
}

if (!defined("_PG65_ACCESS")) {
 define("_PG65_ACCESS", 1);

if (!defined("_DBACCESS")) {
	include ("$inc_dir/dbaccess.$ext");
}


class postgres65_access extends DBaccess {

/* private: link and query handles */
var	$Host     = "";
var	$Database = "";
var	$User     = "";
var	$Password = "";

var	$dblink    = 0;
var	$Query_ID;
var	$Auto_Free = 0;     ## Set to 1 for automatic mysql_free_result()

var	$site;
var	$dbname;


/**
 * Description
 * @var
 * @since     1.0
 * @access    private
 */
var	$sequence_table = "";

#	------------------------------------------------------------------
#				General Database access implementation (cf  : phplib)
#				Copyright (c) 1998,1999 SH Online Dienst GmbH
#					 Boris Erdmann, Kristian Koehntopp
#	------------------------------------------------------------------

function ifadd($add, $me) {
	if("" != $add) return " ".$me.$add;
}

function connect() {
	if ( 0 == $this->dblink ) {
		$cstr = "dbname=".$this->Database.
		$this->ifadd($this->Host, "host=").
		$this->ifadd($this->Port, "port=").
		$this->ifadd($this->User, "user=").
		$this->ifadd($this->Password, "password=");
		$this->dblink=pg_pconnect($cstr);
		if (!$this->dblink) {
			$this->halt("Link-ID == false, pconnect failed");
		}
	}
}

/* public: perform a query */
function query($Query_String) {
	$this->Query_ID = pg_Exec($this->dblink, $Query_String);
	$this->Row   = 0;

	$this->Error = pg_ErrorMessage($this->dblink);
	$this->Errno = ($this->Error == "")?0:1;
	if (!$this->Query_ID) {
		$this->halt("Invalid SQL: ".$Query_String);
	}

	return $this->Query_ID;
}

/* public: walk result set */
function next_record() {
	$this->Record = @pg_fetch_array($this->Query_ID, $this->Row++);

	$this->Error = pg_ErrorMessage($this->dblink);
	$this->Errno = ($this->Error == "")?0:1;

	$stat = is_array($this->Record);
	if (!$stat && $this->Auto_Free) {
		pg_freeresult($this->Query_ID);
		$this->Query_ID = 0;
	}
	return $stat;
}

/* public: position in result set */
function seek($pos = 0) {
	$this->Row = $pos;
}

function affected_rows() {
	return pg_cmdtuples($this->Query_ID);
}

function num_rows() {
	return pg_numrows($this->Query_ID);
}

function num_fields() {
	return pg_numfields($this->Query_ID);
}

#	------------------------------------------------------------------
#				End of PHPLIB copyright
#	------------------------------------------------------------------

#	------------------------------------------------------------------
#				General database access functions
#	------------------------------------------------------------------

function nextId($seqname) {

	$sequence = $seqname . "_seq";
	$result = @pg_exec($this->dblink, "SELECT NEXTVAL('$sequence') AS nextid");

	if (!$result) {
		$result = @pg_exec($this->dblink, "CREATE SEQUENCE $sequence");
		if (!$result) {
			$this->halt("<BR> nextid() function - unable to create sequence<br> : " . pg_ErrorMessage() );
		}
		$result = @pg_exec($this->dblink, "SELECT NEXTVAL('$sequence') AS nextid");
		if (!$result) {
			$this->halt("<BR> nextid() function - unable to create sequence<br> : " . pg_ErrorMessage() );
		}
	}

	$row = @pg_fetch_array($result, 0);
	$this->autoID[$seqname] = $row["nextid"];
	return $this->autoID[$seqname];
}

function openDB ($dbhost="", $dbport="", $dbuser="", $dbpassword="", $dbname="agora", $site="", $persistent=0) {

	if (!function_exists('pg_connect') ) {
		$this->halt(sprintf (ERROR_DB_NOT_SUPPORTED, 'postgreSQL'));
		return ERR_ACCESS;
	}

	$login_string = "dbname=$dbname";

	if (!empty($dbhost)) {
		$this->Host = $dbhost;
		$login_string .= " host=$dbhost";
	}

	if (!empty($dbport)) {
		$this->Port = $dbport;
		$login_string .= " port=$dbport";
	}

	if (!empty($dbuser)) {
		$this->User = $dbuser;
		$login_string .= " user=$dbuser";
	}

	if (!empty($dbpassword)) {
		$this->Password = $dbpassword;
		$login_string .= " password=$dbpassword";
	}

	if ($persistent == 1) {
		$this->dblink = pg_pconnect ($login_string);
	} else {
		$this->dblink = pg_connect ($login_string);
	}

	if (empty($this->dblink)) {
		return -3;
	}

	if (!pg_dbname($this->dblink)) {
		echo "Postgres error: ".pg_errormessage($this->dblink)."<BR>";
		return -3;
	}

	$this->dbname = $dbname;
	$this->site = $site;
	return 0;

}

function closeDB () {
	if (!empty($this->dblink)) {
		return pg_close($this->dblink);
	}
	return TRUE;
}

function createTable ($table, $field_defs) {

	$tmp = $this->Halt_On_Error;
	$this->Halt_On_Error = "report";

# Check if table exists
# ---------------------
	$result = pg_exec($this->dblink, "select relname from pg_class where relname not like 'pg_%'");
	if (!$result) {
		echo "Postgres error: ".pg_errormessage($this->dblink)."<BR>";
		return (-3);
	}
	$count = pg_NumRows ($result);
	$i=0;
	while ($i < $count) {
		if ($table == pg_result ($result, $i, 0)) {
			$this->Halt_On_Error = $tmp;
			return -1;  /*  table already exist */
		}
		$i++;
	}

# Create Table
# ------------
	if (!is_array ($field_defs)) {
		$this->Halt_On_Error = $tmp;
		return -4;
	}

	reset($field_defs);
	while (list($col, $size) = each($field_defs)) {
		if (ereg ("^[0-9+]+$", $size) ) {
			$desc .= ", $col VARCHAR($size) NOT NULL DEFAULT ''";
		} else {
			$type = strtoupper($size);
			switch ($type) {
				case "INT":
					$desc .= ", $col INT NOT NULL DEFAULT 0";
					if ($col == "cle") {
						$desc .= " PRIMARY KEY";
					}
					if ( ($col == 'thread') || ($col=='parent') || ($col == 'newest') ) {
						$indexes[] = $col;
					}
					break;
				case "AUTO":
					$desc .= ", $col INT";
					break;
				case "TIMESTAMP":
					$desc .= ", $col TIMESTAMP DEFAULT NOW()";
					break;
				default:
					$desc .= ", $col $type";
					break;
			}
		}
	}

	$desc = ereg_replace ("^, ", "", $desc);
	$ret = $this->query ("CREATE TABLE $table ($desc)");
	if (!$ret) {
		$this->Halt_On_Error = $tmp;
		return -3;
	}

	if (is_array($indexes) ) {
		reset ($indexes);
		while ( list(,$col) = each($indexes) ) {
			$index = $table."_".$col."_idx";
			$this->query ("CREATE INDEX $index ON $table($col)");
		}
	}

	$this->Halt_On_Error = $tmp;
	return 0;
}

function updateTable ($table, &$fields, $where="") {

	if (!is_array ($fields) ) {
		echo "warning: updateTable(): item to update is not an array";
		return -3;
	}

 #  gets fields name and type from table
 #  ------------------------------------
	$result = pg_exec ($this->dblink,
	 "select a.attname, t.typname from pg_class c, pg_attribute a, pg_type t where c.relname = '$table' and a.attnum > 0 and	a.attrelid = c.oid and a.atttypid = t.oid");
	if (!$result) { /* access problem */
		$this->halt("could not list fields from $table");
		return -3;
	}

	$i=pg_numrows($result);
if ($this->debug) echo "numfields: $i <br>";

#	build columns/values list (set all matching column names)
#	---------------------------------------------------------
	while ($i >= 0) {
		$colname = @pg_result($result, $i, 0);
		$coltype = @pg_result($result, $i, 1);
		if (!empty($colname) && (isset($fields[$colname])) ) {
			if ($coltype == "int4") {
				$values .= ", $colname=" . intVal($fields[$colname]);
			} else {
				$val = addSlashes($fields[$colname]);
				$values .= ", $colname='$val'";
			}
		}
		$i--;
	}
	$values = ereg_replace ("^, ", "", $values);

	if (empty($values) ) {
		echo "warning: updateTable(): no value to update";
		return -3;
	}

	$q = "UPDATE $table SET $values";
	if (!empty($where) ) {
		$q .= " WHERE $where";
	}

if ($this->debug) printf("Debug: query = %s<br>\n", $q);

	$this->Query_ID = pg_Exec($this->dblink, $q);
	if (!$this->Query_ID) {
		echo "Postgres error: ".pg_errormessage()."<BR>";
		return -3;
	}

	return 0;
}

function insertRow ($table, &$fields) {

	if (!is_array ($fields) ) {
		echo "warning: insertRow(): item to insert is not an array";
		return -3;
	}

 #  gets fields name and type from table
 #  ------------------------------------
	$result = pg_exec ($this->dblink, "select a.attname, t.typname from pg_class c, pg_attribute a, pg_type t where c.relname = '$table' and a.attnum > 0 and a.attrelid = c.oid and a.atttypid = t.oid");

	if (!$result) { /* access problem */
		$this->halt("could not list fields from $table");
		return -3;
	}

	$i=pg_numrows($result);
	if ($this->debug) echo "numfields: $i <br>";

#	build columns/values list (set all matching column names)
#	---------------------------------------------------------
	$cols = "";
	$values = "";
	while ($i >= 0) {
		$colname = @pg_result($result, $i, 0);
		$coltype = @pg_result($result, $i, 1);
		if (!empty($colname) && (isset($fields[$colname])) ) {
			$cols .= ", $colname";
			if ($coltype == "int4") {
				$values .= ", " . IntVal($fields[$colname]);
			} else {
				$val = addSlashes($fields[$colname]);
				$values .= ", '$val'";
			}
		}
		$i--;
	}

	if (empty($values) ) {
		echo "warning: insertRow(): no value to update";
		return -3;
	}

	$cols = ereg_replace ("^, ", "", $cols);
	$values = ereg_replace ("^, ", "", $values);

	$q = "INSERT INTO $table ($cols) VALUES($values)";

if ($this->debug) printf("Debug: query = %s<br>\n", $q);

	$this->Query_ID = pg_Exec($this->dblink, $q);
	if (!$this->Query_ID) {
		echo "Postgres error: ".pg_errormessage()."<BR>";
		return -3;
	}

	return 0;
}


#	------------------------------------------------------------------
#				LOG / STATS functions
#	------------------------------------------------------------------


/**
 * Gets monthly statistics.
 *
 * Detail description
 * @param	site	the site 
 * @param	year	the year (YYYY) facultative
 * @param	month	the month (1-12) facultative (if set, year must also be set)
 * @since	4.1.3
 * @access	public
 * @return	array		A double dimension array : $array[month][action] = count 
 *						where month as the form YYYYMM, and action 
 *						can be one value of view|insert|subscribe|login
 */
function getMonthlyStats ($site, $year='', $month='') {
	$log_table = $site.'_log';

	$query = "SELECT extract(year from datestamp) AS y, extract(month from datestamp) AS m, action, count(action) AS cnt FROM $log_table";
	
	if (!empty($year)) {
		$y = sprintf('%04d', $year);
	    $query .= " WHERE extract(year from datestamp)='$y'";
		if (!empty($month)) {
			$m = sprintf('%02d', $month);
			$query .= " AND extract(month from datestamp)='$m'";
		}
	}

	$query .= ' GROUP BY extract(year from datestamp), extract(month from datestamp), action';

	$this->query($query);
	while ($this->next_record()) {
		$m = sprintf ('%04d%02d', $this->Record['y'], $this->Record['m']);
		$action = $this->Record['action'];
		$stats[$m][$action] = (int) $this->Record['cnt'];
	}
	return $stats;
}

/**
 * Gets monthly visitors.
 *
 * Returns an array indexed by months with the number of distinct visitors 
 * @param	site	the site 
 * @since	4.1.3
 * @access	public
 * @return	array		An array of the form: $array[month] = visits where month as the form YYYYMM
 */
function getMonthlyVisitors ($site, $year='', $month='') {
	$log_table = $site.'_log';

	$query = "SELECT extract(year from datestamp) AS y, extract(month from datestamp) AS m, count(DISTINCT userid) as visits FROM $log_table";

	if (!empty($year)) {
		$y = sprintf('%04d', $year);
	    $query .= " WHERE extract(year from datestamp)='$y'";
		if (!empty($month)) {
			$m = sprintf('%02d', $month);
			$query .= " AND extract(month from datestamp)='$m'";
		}
	}

	$query .= ' GROUP BY extract(year from datestamp), extract(month from datestamp)';
	$this->query($query);
	while ($this->next_record()) {
		$m = sprintf ('%04d%02d', $this->Record['y'], $this->Record['m']);
		$visits[$m] = (int) $this->Record['visits'];
	}
	return $visits;
}

/**
 * Gets Daily statistics.
 *
 * Detail description
 * @param	site	the site 
 * @param	year	the year (YYYY) 
 * @param	month	the month (1-12)
 * @since	4.1.3
 * @access	public
 * @return	array		A double dimension array : $array[day][action] = count 
 *						where day is in the rang 1..31 and action can be one value of view|insert|subscribe|login
 */
function getDailyStats ($site, $year, $month) {

	$log_table = $site.'_log';
	$y = sprintf('%04d', $year);
	$m = sprintf('%02d', $month);

	$this->query("SELECT extract(day from datestamp) AS D, action, count(action) AS cnt FROM $log_table WHERE extract(year from datestamp)='$y' AND extract(month from datestamp)='$m' GROUP BY extract(day from datestamp), action");
	while ($this->next_record()) {
		$d = sprintf('%02d', $this->Record['D']);
		$action = $this->Record['action'];
		$stats[$d][$action] = (int) $this->Record['cnt'];
	}
	
	return $stats;
}

/**
 * Gets Daily statistics.
 *
 * Detail description
 * @param	site	the site 
 * @param	year	the year (YYYY) 
 * @param	month	the month (1-12)
 * @since	4.1.3
 * @access	public
 * @return	array		A double dimension array : $array[day][action] = count 
 *						where day is in the rang 1..31 and action can be one value of view|insert|subscribe|login
 */
function getDailyVisitors ($site, $year, $month) {

	$log_table = $site.'_log';
	$y = sprintf('%04d', $year);
	$m = sprintf('%02d', $month);

	$this->query("SELECT extract(day from datestamp) AS D, count(DISTINCT userid) as visits FROM $log_table WHERE extract(year from datestamp)='$y' AND extract(month from datestamp)='$m' GROUP BY extract(day from datestamp)");
	while ($this->next_record()) {
		$d = sprintf('%02d', $this->Record['D']);
		$visits[$d] = (int) $this->Record['visits'];
	}
	return $visits;
}

#	------------------------------------------------------------------
#				Messages (notes) management functions
#	------------------------------------------------------------------

function listThreads ($forum, $first=0, $last=0, $where="", $expanded=1, $limit=0) {
	global $list_var;

	if (empty($forum)) {
		echo "DBaccess error: listThreads() : forum not defined";
		return -3;
	}

	settype ($first, "integer");
	settype ($last, "integer");
	settype ($limit, "integer");

#	1) gets total count # of notes satisfying the query
#	---------------------------------------------------
	$whereClause = "hidden=0";
	if (!empty($where)) {
		$whereClause .= " AND $where";
	}
	$result = @pg_exec ($this->dblink, "SELECT count(*) as total FROM $forum WHERE $whereClause");
	if (!$result) {
		echo "Postgres error: ".pg_errormessage()."<BR>";
		return -3;
	}

	$ret["total"] = @pg_result ($result, 0, "total");
	if ($ret["total"] == 0) {
		return $ret;
	}
	pg_FreeResult($result);

#	2) Find the first (and last) thread to be displayed in the page
#	and gets number of notes per thread
#	----------------------------------------------------------------
	if (!$last) {
		$order="DESC";
		$last=1;
	} else {
		$order="ASC";
	}

	$whereClause = "newest>=$last"; // $last default to 1: don't display key 0
	if ($first>0) {
		$whereClause .= " AND newest<=$first";
	}

	if (!empty($where)) {
		$whereClause .= " AND $where";
	}

# Set # of threads to be returned
# -------------------------------
	$limitClause = (empty($limit)) ? "" : "LIMIT $limit";

	$query = "SELECT thread, newest, count(*) FROM $forum WHERE $whereClause GROUP BY newest,thread ORDER BY newest $order $limitClause";
if ($this->debug) printf("Debug: query = %s<br>\n", $query);
	$result = pg_exec ($this->dblink, "$query");
	$tcount = pg_numrows ($result);

	if ($tcount > 0) {
		if ($order=="DESC") {
			$tfirst  = @pg_result ($result, 0, "newest");
			$tlast = @pg_result ($result, $tcount-1, "newest");
		} else {
			$tlast  = @pg_result ($result, 0, "newest");
			$tfirst = @pg_result ($result, $tcount-1, "newest");
		}

		for ($i=0; $i<$tcount; $i++) {
			$row = pg_fetch_row($result, $i);
			$t = $row[0];
			$replies[$t] = $row[2] - 1;
		}
	}
	pg_FreeResult($result);

	$ret["threads"]=$tcount;    // number of threads displayed
	$ret["first"]=$tfirst;
	$ret["last"]=$tlast;

	if ($tcount < 1) {
		$ret["next"] = 0;
		$ret["prev"] = 0;
		return $ret;
	}

#	3) gets all threads
#	-------------------
	$whereClause = "newest<=$tfirst AND newest>=$tlast";
	if (!empty($where) ) {
		$whereClause .= " AND $where";
	}

	if ($expanded!=1) {
		$whereClause .= " AND parent=0";
	}

	$cols = "cle,parent,childs,thread,newest";
	reset($list_var);
	while (list($var,$val) = each($list_var)) {
		if ($val==1) $cols.=",$var";
	}

	$query = "SELECT $cols FROM $forum WHERE $whereClause ORDER BY newest DESC,unixdate ASC";
if ($this->debug) printf("Debug: query = %s<br>\n", $query);
	$result = @pg_exec ($this->dblink, $query);
	if (!$result) {
		echo "Postgres error: ".pg_errormessage()."<BR>";
		return (-3);
	}

	reset ($this->entries);
	reset ($this->children);
	if (isset($this->children[0]) ) {
		unset($this->children[0]);
	}
	$ncount=0;
	while ($row = @pg_fetch_array($result, $ncount)) {
		$key = (int) $row["cle"];
		$parent = (int) $row["parent"];
		if ($parent==0) {
			$row["replies"] = $replies[$key];
		}
		$this->entries[$key] = $row;
		$this->children[$parent][] = $key;
		$ncount++;
	}
	$ret["notes"]=$ncount;  // number of notes displayed
	pg_freeresult($result);

#	4) gets next/previous count
#	---------------------------
	$whereClause = "hidden=0 AND newest>$tfirst";
	if (!empty($where)) {
		$whereClause .= " AND $where";
	}
	$result = @pg_exec ($this->dblink, "SELECT count(*) AS prev FROM $forum WHERE $whereClause");
	if (!$result) {
		echo "Postgres error: ".pg_errormessage()."<BR>";
		return (-3);
	}
	$ret["prev"] = @pg_result ($result, 0, "prev");
	pg_freeresult($result);

	$whereClause = "hidden=0 AND newest<$tlast";
	if (!empty($where)) {
		$whereClause .= " AND $where";
	}
	$result = @pg_exec ($this->dblink, "SELECT count(*) AS next FROM $forum WHERE $whereClause");
	if (!$result) {
		echo "Postgres error: ".pg_errormessage()."<BR>";
		return (-3);
	}
	$ret["next"] = @pg_result ($result, 0, "next");

	pg_FreeResult($result);
	return $ret;
}

function listNotes ($forum, $first=0, $last=0, $sort="", $where="", $limit) {
	global $list_var;

	if (empty($forum)) {
		echo "DBaccess error: listNotes() : forum not defined";
		return -3;
	}

	settype ($first, "integer");
	settype ($last, "integer");
	settype ($limit, "integer");

#	1) gets total count # of notes satisfying the query
#	---------------------------------------------------
	// skip administrative note (old format)
	// don't skip hidden notes (listed in moderate_notes.php)
	$whereClause = "cle<>0";
	if (!empty($where)) {
		$whereClause .= " AND $where";
	}
	$result = @pg_exec ($this->dblink, "SELECT count(*) as total FROM $forum WHERE $whereClause");
	if (!$result) {
		echo "Postgres error: ".pg_errormessage()."<BR>";
		return -3;
	}

	$ret["total"] = @pg_result ($result, 0, "total");
	if ($ret["total"] == 0) {
		return $ret;
	}
	pg_FreeResult($result);

#	2) gets all notes sorted by $sort
#	---------------------------------
	$whereClause = "WHERE cle>0"; // skip administrative record
	if (!empty($where)) {
		$whereClause .= " AND $where";
	}

	$orderClause = (empty ($sort)) ? "ORDER BY unixdate DESC" : "ORDER BY $sort";

# Set first thread and # of threads to be returned
# ------------------------------------------------
	if ($limit>0) {
		if ($last > 0) {
			$start = ($last-$limit)+1;
			$start = ($start<0) ? 0 : $start;
		} else {
			$start = $first;
		}
		$limitClause = "limit $start, $limit";
	} else {
		$start=0;
		$limitClause="";
	}

	$cols = "cle,parent,childs,thread,newest";
	reset($list_var);
	while (list($var,$val) = each($list_var)) {
		if ($val==1) $cols.=",$var";
	}

	$query = "SELECT $cols FROM $forum $whereClause $orderClause";
	$result = @pg_exec ($this->dblink, $query);
	if (!$result) {
		echo "Postgres error: ".pg_errormessage()."<BR>";
		return (-3);
	}

	reset ($this->entries);
	reset ($this->children);
	$tcount=0;
	while ($row = @pg_fetch_array($result, $tcount)) {
		$key = $row["cle"];
		$row["parent"]=0;
		$this->entries[$key] = $row;
		$this->children[0][] = $key;
		$tcount++;
	}
	pg_FreeResult($result);

	$ret["notes"] = $tcount;
	$ret["threads"] = $tcount;
	$ret["first"] = $start;
	$ret["last"] = $start + $tcount - 1;

#	3) set next/previous (1 if TRUE)
#	---------------------------------
	$ret["next"] = ($ret["total"] > $ret["last"]+1) ? 1 : 0;
	$ret["prev"] = ($ret["first"] > 1) ? 1 : 0;

	return $ret;
}

}; // end class

} // defined _PG65_ACCESS

?>
